/** \file TocBaseEngine.h
 * \brief Declaration of TocBaseEngine class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCBASEENGINE_H
#define TOCBASEENGINE_H

#include "defs.h"

QT_BEGIN_NAMESPACE

/** \brief TocBaseEngine class
 * 
 * Abstract class representing the engine's public interface
 * this interface is used by TocBaseUi
 */
class TocBaseEngine : public QObject
{
	Q_OBJECT

public:

	/**
	 * Public Constructor
	 */
	TocBaseEngine( QObject * parent = 0 );

	/**
	 * Destructor
	 */
	~TocBaseEngine();

signals:

	/** \brief Notifies about connection status change
	 * 
	 * Useful when the server disconnects us
	 * without our request to show the information to the user.
	 * 
	 * @param status Connection status
	 * @param reason Status change reason
	 */
	void statusChanged(Status status, Reason reason);
	
	/** \brief Notifies about user presence change
	 * 
	 * Used to confirm presence change request.
	 * 
	 * @param presence User presence
	 * @param description Presence description
	 */
	void presenceUpdated(Presence presence, QString description);

	/** \brief Notifies about user presence change
	 * 
	 * @param uid Contact's user ID
	 * @param presence Contact's presence
	 * @param description Presence description
	 */
	void contactPresenceUpdated(QString uid, Presence presence, QString description);

	/** \brief Notifies about incoming messages
	 * 
	 * Emited when there is a new pending message to be handled.
	 * 
	 * @param uid Contact's user ID
	 * @param message Message to be handled
	 */
	void incomingMessage(QString uid, Message message);
	
	/** \brief Notifies if there is a new ContactList ready 
	 * 
	 * Full contact list is sent only on TocEngine's startup,
	 * then it sends only the elements that have changed. ( TODO: to be discussed )
	 * 
	 * @param contactList New contact list to be displayed
	 */
	void contactListReceived(const TocContactList& contactList);

	/** \brief Notifies about accepted subscription
	 * 
	 * This signal tells us if the contact we wanted to add
	 * to our contact list, authorized our request.
	 * 
	 * @param uid ID of the contact that accepted our subscription request
	 */
	void subscriptionAccepted(QString uid);

	/** \brief Notifies about authorization request from contact
	 * 
	 * This signal tells us that there is a contact that requested
	 * our authorization to let him add our ID to his contact list.
	 * 
	 * @param uid ID of a contact that requested our authorization
	 */
	void authorizationRequest(QString uid);

public slots:

	/** \brief Used to initialize engine's parameters on startup 
	 * 
	 * This slot also needs to be called when account or connection settings have changed.
	 */
	virtual void initializeFromSettings() = 0;

	/** \brief Used to handle status change request from the UI
	 * 
	 * @param presence Requested presence
	 * @param description Requested description
	 */
	virtual void onPresenceChange(Presence presence, QString description) = 0;
	
	/** \brief Closes the channel(s) created for the given contact
	 * 
	 * When user wants to quit the conversation this slot should be called.
	 * 
	 * @param uid User ID of the contact
	 */
	virtual void onSessionClosed(QString uid) = 0;

	/** \brief Called when User wants to send a message to the contact
	 * 
	 * @param uid Contact's ID
	 * @param message Message to be sent
	 */
	virtual void onNewMessageReady(QString uid, Message message) = 0;

	/** \brief Used to add contact to the contact list
	 * 
	 * @param pContact	Pointer to a contact to be added.
	 */
	virtual void onAddContactRequest(TocContact *pContact) = 0;

	/** \brief Used to edit existing contact data
	 * 
	 * We also allow user to change the ID of the contact.
	 * 
	 * @param pContact	Pointer to a contact to be altered.
	 * @param uid	Contact's old ID
	 */
	virtual void onEditContactRequest(TocContact *pContact, QString oldUid) = 0;

	/** \brief This slot removes a contact from contact list
	 * 
	 * @param uid Contact's ID
	 */
	virtual void onRemoveContactRequest(QString uid) = 0;

	/** \brief Used when we want to change contacts subscription to our account 
	 *
	 * If we want to grant our authorization to the contact's subscritpion request,
	 * set the `authorized` flag to true,
	 * if we do not want the contact to have subscription to our account
	 * set the `authorized` flag to false.
	 * 
	 * @param uid ID of a contact which sibscription we want to change
	 * @param bAuthorized Authorization status
	 */
	virtual void onSubscriptionAccepted(QString uid, bool bAuthorized) = 0;
};

QT_END_NAMESPACE

#endif // TOCBASEENGINE_H
