/** \file TocBaseEngine.h
 * \brief Declaration of TocBaseEngine class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCENGINE_H
#define TOCENGINE_H

#include "TocBaseEngine"
#include <QtTapioca/Connection>
#include <QtTapioca/TextChannel>

namespace QtTapioca {
	class ContactBase;
};

using namespace QtTapioca;

QT_BEGIN_NAMESPACE

/** \brief TocEngine class
 * 
 * Wrapper around the QtTapioca/QtTelepathy communication framework,
 * responsible for managing opened channels and connection(s),
 * and for synchronizaion beetween our contact list and the one we get from the telepathy.
 */
class TocEngine : public TocBaseEngine
{
	Q_OBJECT

public:

	/**
	 * Public Constructor
	 */
	TocEngine( QObject * parent = 0 );

	/**
	 * Destructor
	 */
	~TocEngine();

signals:
	
public slots:

	/** \brief Reimplemented from TocBaseEngine
	 * 
	 */	
	void onPresenceChange(Presence presence, QString description);

	/** \brief Reimplemented from TocBaseEngine
	 * 
	 */	
	void onAddContactRequest(TocContact* pContact);

	/** \brief Reimplemented from TocBaseEngine
	 * 
	 */	
	void onEditContactRequest(TocContact* pContact, QString oldUid);

	/** \brief Reimplemented from TocBaseEngine
	 * 
	 */	
	void onRemoveContactRequest(QString uid);

	/** \brief Reimplemented from TocBaseEngine
	 * 
	 */	
	void onSubscriptionAccepted(QString /*uid*/, bool /*bAuthorized*/) {}
	
	/** \brief Reimplemented from TocBaseEngine
	 * 
	 */
	void initializeFromSettings();

	/** \brief Called when User wants to send a message to the contact
	 * 
	 * @param uid Contact's ID
	 * @param message Message to be sent
	 */
	virtual void onNewMessageReady(QString uid, Message message);

	/** \brief Closes the channel(s) created for the given contact
	 * 
	 * When user wants to quit the conversation this slot should be called.
	 * 
	 * @param uid User ID of the contact
	 */
	virtual void onSessionClosed(QString uid);

private slots:

	/** \brief Used when connection status was changed
	 * 
	 * @param pConnection Handler to QtTapioca connection
	 * @param status Status of connection
	 * @param reason Reason of status change 
	 */
	void onStatusChange(QtTapioca::Connection *pConnection, QtTapioca::Connection::Status status, QtTapioca::Connection::Reason reason);
	
	/** \brief Used when contact presence was changed
	 *
	 * @param pContact Contact item which changed its presence  
	 * @param presence Actual contact's presence
	 * @param presenceMessage Actual contact's presence message
	 */
	void onPresenceUpdated(QtTapioca::ContactBase *pContact, QtTapioca::ContactBase::Presence presence, const QString &presenceMessage);

	/** \brief Used when user presence was changed
	 *
	 * @param pContact Contact item which changed its presence
	 * @param presence Actual users's presence
	 * @param presenceMessage Actual users's presence message
	 */
	void onSelfPresenceUpdated(QtTapioca::ContactBase *pContact, QtTapioca::ContactBase::Presence presence, const QString &presenceMessage);

	/** \brief Used when user receives message
	 *
	 * @param pTextChannel Channel from whom we recieved message
	 * @param message received message
	 */
	void onMessageReceived(const QtTapioca::TextChannel* pTextChannel, const QtTapioca::TextChannel::Message &message);

	/** \brief Used when message has not been sent
	 * 
	 * When an error occured this slot prepares a Message with a proper 'error' field value
	 * 
	 * @param message Message for which an error occured
	 * @param error Error type
	 */
	void onMessageDeliveryError(const QtTapioca::TextChannel::Message &message, QtTapioca::TextChannel::Message::DeliveryError error);

	/** \brief Called when someone wants to initiate chat with user
	 * 
	 * To receive message that initiated this channel one should call pendingMessages method
	 * 
	 * @param pConnection Connection on which new channel is created
	 * @param pChannel Newly created channel
	 * @param bSuppresHandler Used to notify other channel handlers that they should not use this channel object
	 */
	void onChannelCreated(QtTapioca::Connection* pConnection, QtTapioca::Channel* pChannel, bool bSuppresHandler);


	/** \brief Authorizes a contact and add it to users list
	 * 
	 * TODO: this should be implemented in a way that authorization
	 * depends on user settings
	 * 
	 * @param pContact Contact to be authorized
	 */
	void onAuthorizationRequested(QtTapioca::Contact *pContact);
	
private: //Methods

	/** \brief Synchronizes server and local contact lists
	 * 
	 * Lists should be synchronized according to 
	 * the synchronization priority set by user. (NO LONGER TRUE AS THE SYNC PRORITY WILL BE REMOVED)
	 */
	void synchronizeContactLists();

	/** \brief Adds pending contacts to the list on the server
	 * 
	 */
	void addPendingContacts();

	/** \brief Removes pending contacts from the list on the server
	 * 
	 */
	void removePendingContacts();

	/** \brief  Syncronizes contacts from server to the local list
	 * 
	 * When contacts from server are edited in a different client
	 *
	 * Note that this function SHOULD NOT be run alone,
	 * but ONLY AFTER addition and removal of pending contacts from server
	 */
	void syncContactsFromServer();
	
	/** \brief Returns true if a user with the given Uid is already on the list
	 *
	 * @param uid Uid of the searched contact 
	 * @return index of the contact or -1 if not found
	 */
	int indexForUid( QString uid );
	
	/** \brief Removes contact from local contact list
	 * 
	 * @param uid Uid of the contact to be removed
	 */
	void removeContact( QString uid );

	/** \brief Adds contact to the local list
	 * 
	 * @param contact Pointer to the QtTapioca::Contact instance
	 */
	void addContact( const Contact* contact );

private: //Members

	bool								_bNewUserContact;			/// Indicates that this is the first status change after initializeFromSettings
	bool								_bContactListSynchronized; 	/// Indicates if contact list has been already sent to the UI
	QtTapioca::Connection*				_pConnection;				/// Active connection
	TocContactList*						_pTocContactList;			/// Current contact list
	QStringList*						_pAddedContacts;			/// Contacts to be added
	QStringList*						_pRemovedContacts;			/// Contacts to be removed
};

QT_END_NAMESPACE

#endif // TOCENGINE_H
