/** \file IconProvider.h
 * \brief Declaration of IconProvider class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef ICONPROVIDER_H
#define ICONPROVIDER_H

#include <QMap>

#include "defs.h"

QT_BEGIN_NAMESPACE

/** \brief IconProvider class
 * 
 * Purpose of this class is to create centralize way to access icons for contacts.
 * It contains repository of all contact icons both default and custom marged with status frames.
 */
class IconProvider
{

public:

	/** \brief Static public constructor
	 * 
	 * @return Pointer to the static instance
	 */
	static IconProvider* getInstance();

	/**
	 * Destructor
	 */
	~IconProvider();
	
	/** \brief This method finds suitable icon for a given contact and its presence.
	 *
	 * Function looks for an icon inside one of maps _defaultIcons when no custom icon
	 * was set (customIconPath field is empty) and _customIcons when setCustomIcon
	 * function was called for a given contact. 
	 *
	 * @param pContact Contact for which image is requested.
	 * @return Pointer to an image of user's Avatar composed with presence information (green, red or yellow frame)
	 */	
	QImage* matchIcon(const TocContact* pContact) const;

	/** \brief This method finds suitable default icon for a given gender.
	 *
	 * It's convenience function for cases when there's no possibility to set Qt::DecorationRole.
	 *
	 * @param gender Gender for which icon needs to be matched.
	 * @return Default icon for a given gender.
	 */		
	QIcon matchIcon(Gender gender) const;
	
	/** \brief Extends repository of custom icons for a given contact.
	 *
	 * This method adds new set of images to _customIcons map for a contact. It also assigns _customIconId.
	 * Icon id is used later by matchIcon method to access right image. New image is created out of file
	 * from customIconPath.
	 *
	 * @param pContact Contact for which new custom icons are going to be added.
	 */		
	void setCustomIcon(TocContact* pContact);


private: // Methods

	/** \brief Private constructor
	 * 
	 */
	IconProvider();
	
	/** \brief Used to compose presence picture in case presence was changed
	 * 
	 * This method is used to compose picture showed in contact list.
	 * It use contactImage and Presence to create resultImage which is used in contact list.
	 *
	 * @param contactImage Image of user's Avatar
	 * @param presence Presence which will be displayed on resultImage
	 * @return resultImage Image of user's Avatar composed with presence information
	 */
	QImage* addStatusFrame(const QImage* baseImage, Presence presence) const;

private: // Members
	
	static IconProvider* mp_self;
	
	QMap<int,QImage*> 	_customIcons;	///Map of custom images. Created at the begining out of contacts from settings or updated by setCustomIcon. 
	QMap<int,QImage*>	_defaultIcons;	///Map of default images. Created inside the constructor. Used for contacts with no custom icon assign.
	int 			_customIconId;	///This id 'generated' by incrementation when adding new custom icon inside setCustomIcon and enables fast access to images in _customIcons map.
	
	QString _defaultMaleImagePath;
	QString _defaultFemaleImagePath;
	QString _defaultUnknownImagePath;
};

QT_END_NAMESPACE

#endif // ICONPROVIDER_H
