/** \file TocAccountSettingsWidget.h
 * \brief Declaration of TocAccountSettingsWidget class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCACCOUNTSETTINGSWIDGET_H
#define TOCACCOUNTSETTINGSWIDGET_H

#include <QWidget>
#include "defs.h"

class Ui_TocAccountSettingsWidgetClass;

QT_BEGIN_NAMESPACE

/** \brief TocAccountSettingsWidget class
 * 
 * Configuration window for account settings. 
 */
class TocAccountSettingsWidget : public QWidget
{
	Q_OBJECT

public:

	/**
	 * Public Constructor
	 */
	TocAccountSettingsWidget( QWidget * parent = 0, Qt::WindowFlags flags = 0 );

	/**
	 * Destructor
	 */
	~TocAccountSettingsWidget();

	/** \brief Virtual method called when widget is about to be shown
	 * 
	 * Overloaded method used to reload account list, when widget is shown
	 * 
	 * @param type of show event
	 */
	void showEvent(QShowEvent *event);	
	
	/** \brief Checks wheter or not widget is in Ftu mode
	 * 
	 * Returns true if `Create mode` is enabled, otherwise false
	 * 
	 */
	inline bool createMode() const;

	/** \brief Allows enabling/disabling Create mode during runtime
	 * 
	 * Method for switching `Create mode` on and off
	 * 
	 * @param bool create Indicates if widget should enter Create mode
	 */
	void setCreateMode( bool create );

	/** \brief Connects server and protocol hints
	 * 
	 * Used to enable server and protocol hints
	 */
	void connectHints();


signals:

	/** \brief Emited when user clicks 'Ok' button
	 * 
	 * This signal is emited when user changes settings by pressing 'Ok' button
	 * and new settings differs from previous.
	 * Settings of account are already changed, when signal is emitted.
	 * 
	 */
	void accountSettingsChanged();

	/** \brief Emited when user clicks 'Create' button during `Create mode`
	 * 
	 * This signal is emited when user creates settings by pressing 'Create' button.
	 * Settings of account are already saved, when signal is emitted.
	 * 
	 */
	void newAccountSettingsCreated();

	/** \brief Emited when user clicks 'Ok' or 'Create' button
	 * 
	 * This signal is emited when user chooses to close widget by pressing 'Ok' or 'Create' button
	 * regardless if current account was changed.
	 *
	 */
	void finished();

	/** \brief Emited when user wants to go to the previous screen
	 * 
	 */
	void backClicked();

public slots:

	/** \brief Reloads accounts list
	 * 
	 * This method is used to reload account data when it has changed.
	 * 
	 */
	void reload();

private slots:

	/** \brief Commits new data 
	 * 
	 * Verifies data and either triggers creation of new account or stores altered data. 
 	*/
	void onOkButton();

	/** \brief Allows to change password 
	 * 
	 * Shows layout for password change.
 	*/
	void onChangePasswordButton();
	
	/** \brief Connected to textEdited signal from loginLineEdit 
	 * 
	 * Sets text for serverLineEdit and portLineEdit according to new login 
	 * 
	 * @param newLogin new login received from signal
 	*/	
	void loginEdited(const QString& newLogin);

	/** \brief Connected to currentIndexChanged signal from protocolComboBox 
	 * 
	 * Sets text for serverLineEdit and portLineEdit according to new protocol 
	 * 
	 * @param newProtocol new protocol name received from signal
 	*/	
	void protocolEdited(const QString& newProtocol);
	
	/** \brief Disconnects server and protocol hints
	 * 
	 * Used to disable server and protocol hints when user already changed the server
	 * (It would be annoying if hints reset his personaly chosen server and port) 
	 */
	void disconnectHints();

private: // Methods

	/** \brief Displays error message 
	 * 
	 * @param errorText text of error to be displayed
	 */
	void handleError(QString errorText);
	
	/** \brief Validates corectness of entered data
	 * 
	 * @return True if data is corect
	 */	
	bool dataVerified();

	/** \brief Saves edited data
	 * 
	 * If no property was changed it does nothing
	 */
	void saveAccountData();

	/** \brief Creates new account from the data
	 * 
	 */
	void createNewAccount();

private: // Members

	Ui_TocAccountSettingsWidgetClass* ui; 		/// Ui definition header
	bool _bCreateMode;							/// Tells if widget is in `Create mode`
	
};

QT_END_NAMESPACE

#endif // TOCACCOUNTSETTINGSWIDGET_H
