/** \file TocBaseMainWindow.h
 * \brief Declaration of TocBaseMainWindow class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCBASEMAINWINDOW_H
#define TOCBASEMAINWINDOW_H

#include <QtGui/QMainWindow>
#include "defs.h"

class TocBaseEngine;

/** \brief TocBaseMainWindow class
 * 
 * A base class for any Ui class that 
 * wants to comunicate with TocEngine.  
 *
 */
class TocBaseMainWindow : public QMainWindow
{
    Q_OBJECT

public:
    TocBaseMainWindow( TocBaseEngine* engine, QWidget * parent = 0, Qt::WindowFlags flags = 0 );
    ~TocBaseMainWindow();

signals:

	/** \brief Emited when a user wants to change presence
	 * 
	 * This signal is emited when user changes presence in TocMainToolbar.
	 * Note: this is only a request, You should wait for the PresenceUpdated signal from TocEngine
	 * to actualy show to the user his new presence.
	 * 
	 * @param presence Requested presence
	 * @param desc Requested description
	 */
	void presenceChanged(Presence presence, QString desc);

	/** \brief Emited when a session is finished
	 * 
	 * This signal is emited when a TocChatWidget is closed.
	 * 
	 * @param uid Contact's ID
	 */
	void sessionClosed(QString uid);

	/** \brief Emited when a new message is ready to send
	 * 
	 * This signal is emited when user sends a message in one of chatwindows.
	 * 
	 * @param uid Contact's ID
	 * @param message Message to be sent
	 */
	void newMessageReady(QString uid, Message message);

	/** \brief Emited when user adds a new contact
	 * 
	 * This should be emited when user prepares new contact data
	 * using TocUserInfoWidget in "Add mode".
	 * 
	 * @param pContact	Pointer to a contact to be added.
	 */
	void addContact(TocContact* pContact);

	/** \brief Emited when user edits an existing contact
	 * 
	 * This signal is emited when user changes contact data
	 * using TocUserInfoWidget in "Edit mode".
	 * 
	 * @param pContact	Pointer to a contact to be altered.
	 * @param oldUid Previous Contact's ID
	 */
	void editContact(TocContact* pContact, QString oldUid);

	/** \brief Emited when user deletes an existing contact
	 * 
	 * This should be emited when user removes a contact from contact list.
	 * 
	 * @param uid Contact's ID
	 */
	void removeContact(QString uid);

	/** \brief Emited when user creates a new account
	 * 
	 * This signal is emited when user decides to accept or deny request for subscription from someone. 
	 * 
	 * @param uid User ID of the new account
	 * @param authorized Flag that indicates the answer 
	 */	
	void subscriptionAccepted(QString uid, bool authorized);

	/** \brief Emited when user changes the account settings
	 * 
	 */
	void accountSettingsChanged();
	
protected slots:

	/** \brief Slot invoked when someone wants to add us to his contact list
	 *
	 * We just received request for subsciption and need to decide whether accept it 
	 * or discard. This should be connected to the TocEngine's `AuthorizationRequest` signal.
	 * 
	 * @param uid An identificator of a peer that wants to subscribe our account to his contact list
	 */
	virtual void onAuthorizationRequest(QString uid) = 0;
	
	/** \brief Slot invoked when our request for subscription has been accepted
	 * 
	 * This should be connected to the TocEngine's `SubscriptionAccepted` signal.
	 * 
	 * @param uid An identificator of a contact that was requested for subscription  
	 */	
	virtual void onSubscriptionAccepted(QString uid) = 0;
	
	/** \brief Slot handling own status change
	 * 
	 * Thanks this UI can be informed about change in account status.
	 * This should be connected to the TocEngine's `StatusChanged` signal.
	 * 
	 * @param status New status of the account
	 * @param reason The reason of the change
	 */
	virtual void onStatusChange(Status status, Reason reason) = 0;

	/** \brief Slot handling own presence change
	 * 
	 * This should be connected to the TocEngine's `PresenceUpdated` signal.
	 * 
	 * @param presence New presence of the account
	 * @param desc New presence description of the account
	 */
	virtual void onPresenceUpdate(Presence presence, QString desc) = 0;

	/** \brief Slot handling contact's presence change
	 * 
	 * This should be connected to the TocEngine's `ContactPresenceUpdated` signal.
	 * 
	 * @param uid Contact's ID
	 * @param presence New presence of the contact
	 * @param desc New presence description of the contact
	 */
	virtual void onContactPresenceUpdate(QString uid, Presence presence, QString desc) = 0;

	/** \brief Slot handling incoming message
	 * 
	 * This should be connected to the TocEngine's `MessageReceived` signal.
	 * 
	 * @param uid Contact's ID
	 * @param message New message to be handled
	 */
	virtual void onIncomingMessage(QString uid, Message message) = 0;
	
	/** \brief Invoked when new contact list is available
	 * 
	 * This should be connected to the TocEngine's `NewContactList` signal.
	 * 
	 * @param cList New contact list to be used
	 */ 
	virtual void onContactListReceived(const TocContactList& cList) = 0;

};

#endif // TOCBASEMAINWINDOW_H
