/** \file TocChatWidget.h
 * \brief Declaration of TocChatWidget class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCCHATWIDGET_H
#define TOCCHATWIDGET_H

#include <QWidget>

#include "defs.h"

class Ui_TocChatWidgetClass;

QT_BEGIN_NAMESPACE

/** \brief TocChatWidget class
 * 
 * Widget used as a conversation window. Enables sending of messages
 * and notifies about presence changes of the contact by textual information
 * and conversion window border color.
 */
class TocChatWidget : public QWidget
{
	Q_OBJECT

public:
	
	/**
	 * Public Constructor
	 */
	TocChatWidget( const QString& uid, const QString& name, Presence presence, QWidget * parent = 0, const Message& message = Message() );

	/**
	 * Destructor
	 */
	~TocChatWidget();
	
	/** \brief Used to read uid property
	 * 
	 * @return ID of the Contact for which the chat was created   
	 */
	QString uid() const { return _uid; }
	
	/** \brief Used to read name property
	 * 
	 * @return Name of the Contact for which the chat was created   
	 */
	QString name() const { return _name; }
	
	/** \brief Used to set name property
	 * 
	 * @param New name of the Contact   
	 */
	void onDisplayedNameChange(const QString& newName) { _name = newName; }
	
	/** \brief Used to handle contact's presence change
	 * 
	 * This method is used when status of the Contact has changed.
	 * It gives textual and graphical notification to the user.
	 * 
	 * @param presence New contact's presence
	 * @param description New contact's presence description
	 */
	void onContactPresenceUpdate(Presence presence, const QString& description);

	/** \brief Used to update contact's presence
	 * 
	 * This is used when the chat window was closed and did not
	 * recieved Contact Presence Updates, so it can show the current status properly.
	 */
	void updateContactPresence(Presence presence);
	
	/** \brief Used to handle incoming message
	 * 
	 * @param message Message to be displayed in chat
	 */
	void appendMessage(const Message& message);
	
protected:
	
	/** \brief Used to filter incoming events
	 * 
	 * This method is used to filter keyboards event. 
	 * If Enter key is pressed, typed message is send (as pressing Send button).
	 * If Enter key is pressed with Shift key modifier new line in entered.
	 * 
	 * @param *object Pointer to the object on which an event appears.
	 * @param *event Pointer to the event.
	 * 
	 * @return If the event is a key event Enter without Shift modifier 
	 * method returns true, otherwise false.
	 *  
	 */	
	bool eventFilter(QObject *object, QEvent *event);

	/** \brief Reimplemented from QWidget
	 * 
	 */
	virtual void showEvent(QShowEvent* event);

signals:

	/** \brief Emited when new message is ready to be sent
	 *
	 * This signal is emited when user want's to send a prepared message.
	 * 
	 * @param uid Contact's ID
	 * @param message Message to be sent
	 */ 
	void newMessageReady(const QString& uid, Message message);

	/** \brief Emited Smile button clicked.
	 *
	 * This signal is emited when user clicks on Smile button in Chat view,
	 * to choose emoticon.
	 * 
	 */
	void smileButtonClicked();

private slots:

	/** \brief Send message slot
 	*
 	* Invoked when message from chatWidget needs to be sent.
 	* It composes a message, emits a signal and wipes out a text from messageTextEdit.  
 	* 
  	*/
	void pressedSendButton();
	/** \brief Shows emoticons dialog window.
 	*
  	*/	
	void showEmoticonsDialog();
	

private: // Methods

	/** \brief Returns a human readable string for the given error type
	 * 
	 * @param error Error id
	 * @return Human readable error description
	 */
	QString errorString(MessageDeliveryError error);

	/** \brief Appends new message to the chat dialog edit with parsed emoticons
	 * 
	 */
	/** \brief Inserts emoticons.
	 * 
	 * Checks user message text for emoticons aliases and replace them with 
	 * corresponding inage. 
	 *  
	 * @param message Message from user message text edit box.
	 * 
	 */	
	void insertEmoticons(const QString& message);

private: // Members
	
	QString _uid;				/// User ID.
	QString _name;				/// User name.
	
	Ui_TocChatWidgetClass* ui;	/// Pointer to the TocChatWidgetClass form.
};

QT_END_NAMESPACE

#endif // TOCCHATWIDGET_H
