/** \file TocContactListWidget.h
 * \brief Declaration of TocContactListWidget class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCCONTACTLISTWIDGET_H
#define TOCCONTACTLISTWIDGET_H

#include <QtGui/QListWidget>
#include <QAction>
#include <QColor>

#include "defs.h"
#include "TocSettings"

QT_BEGIN_NAMESPACE

/** \brief TocContactListWidget class
 * 
 * Widget representing the list of current account contact's.
 * QListWidget is used in `IconMode`, and allows user to show a unique
 * avatar for each contact. It allows user to delete add and change contact's data,
 * and to start conversation with the contact. 
 */
class TocContactListWidget : public QListWidget
{
	Q_OBJECT
	// Property holding color of the logo.
	// It's a Q_PROPERTY to enable easy setting of the color with style sheet's.
	Q_PROPERTY(QColor logoColor READ logoColor WRITE setLogoColor NOTIFY logoColorChanged)

public:

	/**
	 * Public Constructor
	 */
	TocContactListWidget( QWidget * parent = 0 );

	/**
	 * Destructor
	 */
	~TocContactListWidget();

	/** \brief Method used when contact's presence has changed
	 * 
	 * This method handles presence change of a contact.
	 * 
	 * @param uid Contact's ID
	 * @param presence Contact's new presence
	 * @param description Contact's new presence description
	 */
	void onContactPresenceUpdate(const QString& uid, Presence presence, const QString& description);

	/** \brief This method translates Contact's ID to its Name
	 * 
	 * This method is used when TocMainWindow needs the Name for
	 * the given Uid, in order to create a new TocChatWidget. 
	 * 
	 * @param uid Contact's ID
	 * @return Displayed Name of the contact
	 */
	QString getDisplayedName(const QString& uid);

	/** \brief This method is used to find specific contact on the list by contact's uid
	 * 
	 * It searches for the user in contact list and returns it's item,
	 * if there is no such contact it returns 0. It's possible to exlude particular
	 * item from search. It's used when editing an exisiting contact because contact data
	 * are associated with the same QListWidgetItem pointer.
	 * 
	 * @param item Item to be excluded from searching
	 * @param uid Contact's ID
	 * @param pExcludedItem Item from the list 
	 * @return Pointer to found item
	 */
	QListWidgetItem* findItem(const QString& uid, const QListWidgetItem* pExcludedItem = 0);

	/** \brief Clears statuses of contacts
	 * 
	 * Typicaly used to clear statuses after setting the self-status to Offline
	 */
	void clearStatuses();
	
	/** \brief Getter for logoColor property
	 * 
	 */
	QColor logoColor() const;
	
	/** \brief Setter for logoColor property
	 * 
	 */
	void setLogoColor(QColor rgb);

protected:

	/** \brief Implementation of the interface that allows to handle mouse double click event
	 */
	void mouseDoubleClickEvent(QMouseEvent *event);
	
	/** \brief Implementation of the interface that allows to handle context menu event
	 * 
	 *  It creates context menu for managing contact items on the list.
	 */	
	void contextMenuEvent(QContextMenuEvent *event);
	
	/** \brief Implementation of the interface that allows to handle custom paint events
	 * 
	 *  It allows to paint Tieto logo in contact list view.
	 */	
	void paintEvent(QPaintEvent * event);


signals:

	/** \brief Emited when user chooses a contact to chat with 
	 * 
	 * This signal is emited when user selects and then clicks an item on the list.
	 * 
	 * @param uid Contact's ID
	 * @param name Contact's name
	 * @param presence Contact's presence
	 * 
	 */
	void triggered(const QString& uid, const QString& name, Presence presence);

	/** \brief Emited when user requests to see a contact info 
	 * 
	 * This signal is emited when user selects to edit contact's data
	 * or if the user wants to add new contact to the list ( then *item == QListWidgetItem() ).
	 * 
	 * @param pItem Contact item to be edited or added
	 */
	void contactInfoRequest(QListWidgetItem* pItem);

	/** \brief Emited when contact is removed
	 * 
	 * This signal is emited when user wants to delete the contact with given Uid.
	 * Note that the contact should be already deleted from the TocListWidget
	 * before it's emision. 
	 * 
	 * @param uid Contact's ID
	 */
	void removedContact(const QString& uid);

	/** \brief Emited when user commits the changes
	 * 
	 * This signal is emited when user finished adding contact. 
	 * 
	 * @param pContact	Pointer to a contact to be added.
	 */
	void addContact(TocContact* pContact);
	
	/** \brief Emited when user commits the changes
	 * 
	 * This signal is emited when user finished editing contact data. 
	 * 
	 * @param pContact	Pointer to a contact to be altered.
	 * @param oldUid Previous ID of a contact (omited if widget is in "Add Mode")
	 */
	void editContact(TocContact* pContact, QString oldUid);
	
	/** \brief Emited when name of the contact has been changed 
	 * 
	 * This signal is emited to notify that user chas changed the displayed name
	 * of the contact so it can be used to change the name at runtime
	 * in TocChatWindow and in active sessions menu of TocMainToolbar.
	 * 
	 * @param uid Contact's ID
	 * @param name New displayed name of the contact
	 */
	void displayedNameChanged( const QString& uid, const QString& name);
	
	/** \brief Emited when name of the logoCount property has been changed 
	 * 
	 * When logoCount property is changed, this signal is emited, with new value as a parameter.
	 * 
	 * @param rgb new value of logoColor property
	 */
	void logoColorChanged(QColor rgb);
	
public slots:

	/** \brief Used to set a contact list to the widget
	 * 
	 * This slot is used when a new TocContactList has arived from TocEngine.
	 * Data from the list should be copied to QListWidgetItem's.
	 * 
	 * @param contactList New contact list to be handled
	 */
	void onNewContactList(const TocContactList& contactList);

	/** \brief Used when user finishes editing new contact data
	 * 
	 * This slot is used when user actualy commits new contact's addition
	 * from the TocUserInfoWidget. It adds created item to the contact list.
	 * 
	 * @param pNewItem Item to be added
	 */
	void commitAdd(QListWidgetItem* pNewItem);

	/** \brief Used when user finishes edition of contact data
	 * 
	 * This slot is used when user actualy commits changes to the contact
	 * from the TocUserInfoWidget.
	 * 
	 * @param pEditedItem Item with changed data
	 */
	void commitEdit(QListWidgetItem* pEditedItem);

private slots:

	/** \brief Slot that reacts on Edit action from context menu
	 * 
	 * It extracts QListWidgetItem from sender and posts it to UserInfoWidget
	 * for edition.
	 */
	void onEditRequested();

	/** \brief Slot that reacts on Remove action from context menu
	 * 
	 * It extracts QListWidgetItem from sender, removes it from the contacts list
	 * and notifies engine about it.
	 */	
	void onRemoveRequested();
	
	/** \brief Slot that sets the color of the logo
	 * 
	 * This method paints the logo with the color provided.
	 * 
	 * @param rgb Color to be set for the logo
	 */
	void changeLogoColor(QColor rgb);

private: // Methods

	/** \brief Sorts all the items in the list widget
	 * 
	 * This method sorts all the items in the list widget according to
	 * the specified order.
	 * TODO: only Literal sorting is now implemented, implement other methods as well
	 * 
	 * @param order Order to sort by
	 */
	void sortItems(SortOrder order = Alphabetical);

private: // Members

	TocContact 			_currentEditedContact;	/// Temporary TocContact used in the process of editing an item
	QListWidgetItem* 		_pCurrentEditedItem;	/// Temporary item used in the process of editing an item	
	QAction 			_editAction;		/// An action from context menu that triggers edition of selected item. It's connected to onEditRequested() slot
	QAction 			_removeAction;		/// An action from context menu that triggers deletion of selected item. It's connected to onRemoveRequested() slot
	QImage*				_logo;			/// Logo image mask. Created inside constructor.
	QColor 				_logoColor;		/// Color in RGB of the logo. (Q_PROPERTY logoColor)
};

QT_END_NAMESPACE

#endif // TOCCONTACTLISTWIDGET_H
