/** \file TocMainToolbar.h
 * \brief Declaration of TocMainToolbar class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCMAINTOOLBAR_H
#define TOCMAINTOOLBAR_H

#include <QWidget>
#include <QWidgetAction>
#include <QHBoxLayout>

#include "defs.h"
#include "uidefs.h"

class QPushButton;
class QMenu;
class QAction;
class QLabel;
class ActiveSessions;
class ScrollWidget;

QT_BEGIN_NAMESPACE

/** \brief TocMainToolbar class
 * 
 * Applications  main toolbar used to control window switching, presence changes
 * and management of active chat windows.
 * It also shows current activity name, notifies about incoming messages
 * and allows to quickly switch between converstion and contact list and back.
 * The main toolbar is allways visible in applications window.
 */
class TocMainToolbar : public QWidget
{
	Q_OBJECT

public:

	/**
	 * Public Constructor
	 */
	TocMainToolbar( QWidget * parent = 0, Qt::WindowFlags flags = 0 );

	/**
	 * Destructor
	 */
	~TocMainToolbar();
	
	/** \brief This method is used when new message arives
	 * 
	 * When the message arives, `Active Sessions` button should notify this
	 * by changing color of the button and the corresponding menu action.
	 * If action doesn't exist, it must create it.
	 * 
	 * @param uid Contact's uid
	 * @param name Contact's name
	 * @param presence Contact's current presence
	 */
	void onNewMessage(const QString& uid, const QString& name = "", Presence presence = Unspecified);
	
	/** \brief This method is used when current window changes
	 * 
	 * It is used only if window changes without the knowledge of TocMainToolbar
	 * (i.e. when not triggered by toolbar but by settings window or contact list).
	 * 
	 * @param viewType Type of the window
	 * @param text Text to be displayed in notification area
	 */
	void onCurrentView(ViewType viewType, const QString& text = QString() );
	
	/** \brief This method is used to handle contacts dislayed name changes
	 * 
	 * It changes the name of the corresponding action.
	 * 
	 * @param uid Contact's uid
	 * @param name New contact's name
	 */
	void onDisplayedNameChange(const QString& uid, const QString& name);
	
	/** \brief This method is used to handle contacts presence changes
	 * 
	 * It changes the presence icon of the corresponding action.
	 * 
	 * @param uid Contact's uid
	 * @param name New contact's name
	 */
	void onContactPresenceUpdate(const QString& uid, Presence presence);
	
	/** \brief This method is used to handle account presence changes
	 * 
	 * It should follow the `presenceUpdated` signal if presence change
	 * was succesfull.
	 * 
	 * @param presence Own presence
	 * @param desc Own presence description
	 */
	 void onPresenceUpdate(Presence presence, const QString& desc);

	 
	 /** \brief Used to close all sessions at once
	  * 
	  * Closes all sessions at once on request from outside of the toolbar.
	  * Typicaly useful when user changes account data
	  * and the previous sessions needs to be cleared.
	  */
	 void clearAllSessions();

signals:

	/** \brief Emited when user chooses a conversation window from active sessions menu
	 * 
	 * This signal is emited when user triggers an action in `Active Sessions` menu.
	 * 
	 * @param uid Contact's ID
	 * @param name Contact's name
	 */
	void activeSessionTriggered(const QString& uid, const QString& name);

	/** \brief Emited when user closes the chat window
	 * 
	 * This signal is emited when user chooses to close current chat window.
	 * 
	 * @param uid Contact's ID for which the window was created
	 */
	void finishedSession(const QString& uid);

	/** \brief Emited when user closes all chat windows
	 * 
	 * This signal is emited when user chooses to close all chat windows
	 * using `Close All Sessions` button in `Active Sessions` menu.
	 */
	void finishedAllSessions();

	/** \brief Emited when user chooses contact list
	 * 
	 * This signal is emited when user clicks contacts button.
	 */
	void contactsClicked();

	/** \brief Emited when user chooses settings window
	 * 
	 * This signal is emited when user clicks settings button.
	 */
	void settingsClicked();

	/** \brief Emited when user changes presence 
	 * 
	 * This signal is emited when user requests presence change using
	 * presence button's menu.
	 * 
	 * @param presence Requested presence
	 * @param desc Requested presence description
	 */
	void presenceUpdate(Presence presence, const QString& desc);

public slots:

	/** \brief This slot is used when user requests a new session
	 * 
	 * When new session is started at user request, main toolbar should react for this
	 * by displaying the name of a contact and disabling the corresponding menu action.
	 * If action doesn't exist, main toolbar must create it.
	 * 
	 * @param uid Contact's uid
	 * @param name Contact's name
	 * @param presence Contact's current presence
	 */
	void onNewSession(const QString& uid, const QString& name, Presence presence);

private slots:

	/** \brief Used to close current session
	 * 
	 * Connected to the notification area button when in "Close role".
	 */
	void closeCurrentSession();

	/** \brief Used to close all sessions at once
	 * 
	 * Connected to the close all sessions action, and closes all sessions in one run.
	 */
	void closeAllSessions();

	/** \brief Used to return to the last chat window
	 * 
	 * Connected to the notification area button when in "Return role".
	 */
	void restoreCurrentSession();

	/** \brief Used when user clicks on a name in active sessions menu
	 * 
	 * Handles user clicks in action's from active sessions menu,
	 * it does nothing when action is Close All Sessions action.
	 * 
	 * @param action The action triggered
	 */ 
	void onActiveSessionTriggered(QAction* pAction);

	/** \brief Used when user clicks on a presence in presence button's menu
	 * 
	 * Handles user clicks in action's from presence button's menu.
	 * 
	 * @param action The action triggered
	 */ 
	void onTriggeredPresence(QAction* action);
	
	/** \brief Used to handle `Contacts Button` click action
	 * 
	 * If user clicks `Contacts Button` the toolbar should change
	 * the `Label Button` text to "" and disable it, but if there is chat window active
	 * the `Label Button` should display Name of the Contact the chat is active for,
	 * and change it's mode to "Restore Chat".
	 */
	void onContactsClicked();
	
	/** \brief Used to handle `Settings Button` click action
	 * 
	 * If user clicks `Contacts Button` the toolbar should change
	 * the `Label Button` text to "Settings" and disable it.
	 * It should also disconnect all the connections made for `Label Button`.
	 */
	void onSettingsClicked();

	
private: // Methods
	
	/** \brief Sets style sheets for each widget
	 * 
	 * Used to overload style sheet properties that could come from main style sheet,
	 * to forbid corruption of the application layout from the external style sheets. 
	 */
	void setupStyles();
	
	/** \brief Returns an action with the given Uid
	 * 
	 * If the action is not on the list it retruns 0.
	 * 
	 * @param uid Contact's ID
	 */
	QAction* action( const QString& uid );
	
	/** \bried Highlights the selected button
	 * 
	 * @param button Button to be highlighted
	 */
	void highlightButton(const QPushButton* button);

private: // Members

	QPushButton* 	statusButton;			/// Button for the presence menu
	QPushButton* 	statusAreaButton;		/// Notification area button ("Eduardo button")
	QLabel* 		statusAreaLabel;		/// Notification area label ("Eduardo label")
	QPushButton* 	activeSessionsButton;	/// Button for the active sessions menu
	QPushButton* 	settingsButton; 		/// Settings button
	QPushButton* 	contactsButton;			/// Contact list button
	QMenu*			statusMenu;				/// Status selection menu
	QAction*		onlineAction;			/// Action for online presence
	QAction*		awayAction;				/// Action for away presence
	QAction*		busyAction;				/// Action for busy presence
	QAction*		hiddenAction;			/// Action for hidden presence
	QAction*		offlineAction;			/// Action for offline presence
	
	ActiveSessions* _pActiveSessions;		/// Pointer to an object that manages active sesssions
	
};


/** \brief ActiveSessions class
 * 
 * Class that encapsulates active sessions menu. The main role of this class it to ensure that 
 * there aren't to many actions displayed on the screen at the same time and allows to scroll
 * them. 
 */

class ActiveSessions: public QWidget
{
	Q_OBJECT
	
public:

	/**
	 * Public Constructor
	 */
	ActiveSessions(QWidget * parent = 0);
	
	/** \brief It adds an action to _pActiveSessionsMenu
	 * 
	 * When adding new action this function takes into account how many actions can be shown 
	 * at the same time (_visibleActionsAllowed) and also when to show additional itmes like
	 * _pCloseAllSessionsAction or _pScrollAction.
	 * 
	 * @param pAction action to be added
	 */
	void addAction(QAction *pAction);
	
	/** \brief It removes an action from _pActiveSessionsMenu
	 * 
	 * When removing an action this function takes into account how many actions can be shown 
	 * at the same time (_visibleActionsAllowed) and also when to show additional itmes like
	 * _pCloseAllSessionsAction or _pScrollAction.
	 * 
	 * @param pAction action to be added
	 */
	void removeAction(QAction *pAction);
	
	/** \brief Removes currently selected action from the menu selects a new one and returns a pointer to it
	 * 
	 */
	QAction* closeCurrentSession();

	/** \brief Returns a disabled action
	 * 
	 * If there is no disabled action it retruns 0.
	 */
	QAction* disabledAction() const;
	
	/** \brief Returns an action with the given Uid
	 * 
	 * If the action is not on the list it retruns 0.
	 * 
	 * @param uid Contact's ID
	 */
	QAction* action(const QString& uid) const;
	
	/** \brief Returns a list of an actions from _pActiveSessionsMenu
	 * 
	 */
	QList<QAction*> actions() const;
	
	/** \brief Clears all actions from _pActiveSessionsMenu
	 * 
	 */
	void clear();
	
	/** \brief Returns a pointer to _pActiveSessionsMenu
	 * 
	 */
	QMenu* menu() const {return _pActiveSessionsMenu;} 
	
	/** \brief Handles change of a presence
	 * 
	 * It resets an icon according to new presence
	 * 
	 * @param uid Contact's uid
	 * @param presence New presence 
	 */
	void onContactPresenceUpdate(const QString& uid, Presence presence);
	
	/** \brief Handles new message event
	 * 
	 * It checks for an action for given uid and eventually creats a new one
	 * 
	 * @param uid Uid of a contact for which new message come
	 * @param name Contact's name 
	 * @param presence Contact's presence 
	 */
	void onNewMessage(const QString& uid, const QString& name, Presence presence);
	
	/** \brief This slot is used when user requests a new session
	 * 
	 * This function selects an action with given uid or if it doesn't exist adds it to menu.
	 * 
	 * @param uid Contact's uid
	 * @param name Contact's name
	 * @param presence Contact's current presence
	 */
	void onNewSession(const QString& uid, const QString& name, Presence presence);	
	
private:

	/** \brief Enables or disables scroll up and down buttons according to currently visible actions
	 * 
	 */
	void checkScrollButtonsState(); 

private slots:

	/** \brief Scrolls up actions from _actionsList that are visible in _pActiveSessionsMenu
	 * 
	 */
	void scrollUp();
	
	/** \brief Scrolls down actions from _actionsList that are visible in _pActiveSessionsMenu
	 * 
	 */	
	void scrollDown();
	
private:
	
	QMenu*						_pActiveSessionsMenu;		/// Menu for chat window selection
	QAction*					_pCloseAllSessionsAction;	/// Action to close all active sessions
	QAction*					_pSeparatorAction;			/// Action for separator
	QWidgetAction*				_pScrollAction;				/// Action from _pActiveSessionsMenu that is associated with _pScrollWidget
	ScrollWidget*				_pScrollWidget;				/// Widget with two buttons that enables scrolling
	QList<QAction*> 			_actionsList;				///	List of actions that not includes 'Close all', separator and scroll
	const int					_visibleActionsAllowed;		/// Variable that limits amount of active session actions present on _pActiveSessionsMenu
	QList<QAction*>::iterator	_firstVisibleAction;		/// Iterator that is fixed on first active session action from _pActiveSessionsMenu
	QList<QAction*>::iterator 	_lastVisibleAction;			/// Iterator that is fixed on last active session action from _pActiveSessionsMenu
	
};


/** \brief ScrollWidget class
 * 
 * This small widget was meant as a part of active sessions menu to provide a way to scroll it. 
 */

class ScrollWidget: public QWidget
{
	Q_OBJECT
	
public:
	/**
	 * Public Constructor
	 */
	ScrollWidget(QWidget *parent = 0);
	
	QPushButton* 	scrollUpButton() const { return _pScrollUpButton; }
	QPushButton* 	scrollDownButton() const { return _pScrollDownButton; }	
	
private:
	QPushButton* 	_pScrollUpButton;
	QPushButton* 	_pScrollDownButton;
	QHBoxLayout*	_pLayout;
};


QT_END_NAMESPACE

#endif // TOCMAINTOOLBAR_H
