/** \file TocMainWindow.h
 * \brief Declaration of TocMainWindow class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCMAINWINDOW_H
#define TOCMAINWINDOW_H

#include <QMainWindow>
#include <QListWidgetItem>
#include "TocBaseMainWindow"
#include "defs.h"

class TocBaseEngine;
class QStackedLayout;
class TocChatWidget;
class TocMainToolbar;
class TocContactListWidget;
class TocUserInfoWidget;
class TocSettingsWidget;
class TocGeneralSettingsWidget;
class TocAccountSettingsWidget;
class TocAccountIntermediateWidget;
class TocNetworkSettingsWidget;
class TocThemesSettingsWidget;
class QCloseEvent;

QT_BEGIN_NAMESPACE

/** \brief TocMainWindow class
 * 
 * Class responsible for user GUI management
 * and for communication with TocEngine.
 */
class TocMainWindow : public TocBaseMainWindow
{
	Q_OBJECT
	Q_CLASSINFO("D-Bus Interface", "org.indt.tietoopcom")

public:

	/**
	 * Public Constructor
	 */
	TocMainWindow( TocBaseEngine* engine, QWidget * parent = 0, Qt::WindowFlags flags = 0 );

	/**
	 * Destructor
	 */
	~TocMainWindow();

protected:
	
	/** \brief Reimplemented from QWidget
	 * 
	 */
	virtual void closeEvent(QCloseEvent* event);
	
public slots:
	Q_SCRIPTABLE int top_application();
	
protected slots:

	/** \brief Reimplemented from TocBaseMainWindow
	 * 
	 */
	void onAuthorizationRequest(QString uid);
	
	/** \brief Reimplemented from TocBaseMainWindow
	 * 
	 */
	void onSubscriptionAccepted(QString uid);
	
	/** \brief Reimplemented from TocBaseMainWindow
	 * 
	 */
	void onStatusChange(Status status, Reason reason);

	/** \brief Reimplemented from TocBaseMainWindow
	 * 
	 */
	void onPresenceUpdate(Presence presence, QString desc);

	/** \brief Reimplemented from TocBaseMainWindow
	 * 
	 */
	void onContactPresenceUpdate(QString uid, Presence presence, QString desc);

	/** \brief Reimplemented from TocBaseMainWindow
	 * 
	 */
	void onIncomingMessage(QString uid, Message message);
	
	/** \brief Reimplemented from TocBaseMainWindow
	 * 
	 */
	void onContactListReceived(const TocContactList& cList);	

private slots:

	/** \brief Shows chat window for the given Uid
	 * 
	 * If the chat with Uid is already created, just show it,
	 * if not, create a chat with the given Uid and Name
	 * and emit newSessionStarted(Uid) signal.
	 * Note that this is overloaded SLOT, take caution with that.
	 * 
	 * @param uid Contact's ID
	 * @param name Contact's name
	 * @param presence Contact's current presence
	 */
	void showChat(const QString& uid, const QString& name, Presence presence = Offline);


	/** \brief Closes a chat for the given Uid
	 * 
	 * This slot closes a chat for the given Uid
	 * and emits sessionClosed(Uid) signal to the engine.
	 * 
	 * @param uid Contact's ID
	 */
	void closeChat(const QString& uid);

	/** \brief Closes all active chat windows
	 * 
	 * This slot closes all open chat windows
	 * and emits sessionClosed(Uid) signal for each chat.
	 */
	void closeAllOpenChats();

	/** \brief Closes all active chat windows
	 * 
	 * This slot closes and deletes all created chat windows
	 * and emits sessionClosed(Uid) signal for each chat.
	 * This is typicaly useful when user changes account data.
	 */
	void clearAllChats();

	/** \brief Shows window of a given type
	 * 
	 * Shows a TocWidget of the type == ViewType (except for TocChatWidget)
	 * `ViewType` is static-casted to `int` in a signal emission in the TocSettingsWidget
	 * si it could use a QSignalMapper class.
	 * 
	 * @param type Type of the view to be shown 
	 */
	void showView(/*ViewType*/int type);

	/** \brief Shows TocUserInfoWidget
	 * 
	 * Creates and displays TocUserInfoWidget with the data provided by the signal.
	 * 
	 * @param item Item with contact's data (*item==QListWidgetItem() if You want TocUserInfoWidget to be run in "Add mode")
	 */
	void showUserInfo(QListWidgetItem* item);
	
	/** \brief Shows TocContactListWidget
	 * 
	 * Displays TocContactListWidget.
	 */
	void showContacts();
	
	/** \brief Shows TocContactListWidget
	 * 
	 * Displays TocContactListWidget.
	 * Also informs TocMainToolbar about the change.
	 */
	void returnToContacts();
	
	/** \brief Shows TocSettingsWidget
	 * 
	 * Creates if necessary and displays TocSettingsWidget.
	 */
	void showSettings();
	
	/** \brief Shows TocSettingsWidget
	 * 
	 * Displays TocSettingsWidget .
	 * Also informs TocMainToolbar about the change.
	 */
	void returnToSettings();
	
	/** \brief Shows TocAccountSettingsWidget
	 * 
	 * Creates if necessary and displays TocAccountSettingsWidget.
	 */
	void showAccount();
	
	/** \brief Shows TocAccountIntermediateWidget
	 * 
	 * Creates if necessary and displays TocAccountIntermediateWidget .
	 */
	void returnToIntermediate();

	/** \brief Shows TocAccountSettingsWidget in `Create mode`
	 * 
	 * Creates if necessary and displays TocAccountSettingsWidget.
	 * When user clicks `Create` button the account settings are
	 * saved in TocSettings and the `_register` flag of TocSettings is set to "true".
	 */
	void showAccountCreate();
	
	/** \brief Changes the displayed name in the application on user request
	 * 
	 * If a user changes the displayed name using TocUserInfoView in editing mode
	 * the slot changes the name in the TocContactListWidget and, if needed,
	 * in TocMainToolbar and TocChatWidget.
	 * 
	 * @param uid Contact's ID
	 * @param name Contact's new name
	 */
	void onDisplayedNameChange(const QString& uid, const QString& name);	

	/** \brief Reloads the displayed application theme
	 * 
	 * Reads file URL from settings and sets the stylesheet as the current one.
	 */	
	void reloadTheme();

	/** \brief This slot prepares Toc for the account details change
	 * 
	 * It disconnects, clears the contact list and active sessions list.
	 */
	void onAccountSettingsChanged();

	/** \brief This slot prepares Toc for the new account creation
	 * 
	 * It disconnects, clears the contact list and active sessions list.
	 * Then it connects with the `_register` parameter set to true,
	 * when the connection is successfull the account has been created.
	 */
	void onAccountSettingsCreated();

	/** \brief Shows About info widget
	 * 
	 */
	void showAbout();
	
private: // Methods

	/** \brief Returns cached TocChatWidget for the given Uid
	 * 
	 * If there is a chached chat with the given uid then
	 * it returns pointer to this chat, if not it returns 0.
	 * 
	 * @param uid Contact's ID
	 * @return Pointer to the chached TocChatWidget or 0 if there is no active chat.
	 */
	TocChatWidget* chatForUid(const QString& uid);

	/** \brief Returns active/open TocChatWidget for the given Uid
	 * 
	 * If there is a active/open chat with the given uid then
	 * it returns pointer to this chat, if not it returns 0.
	 * 
	 * @param uid Contact's ID
	 * @return Pointer to the active/open TocChatWidget or 0 if there is no active chat.
	 */
	TocChatWidget* openChatForUid(const QString& uid);

	/** \brief Checks if this is FTU
	 * 
	 * returns true if settings on which FirstTimeUse case is based, are empty
	 * 
	 * @return bool value indicating FTU use case
	 */
	bool isFtu();

	/** \brief Returns an error string for the given error
	 * 
	 * @param reason Reason of the error
	 * @return Human readable error string
	 */
	QString errorString(Reason reason);

private: // Members
	QString 			_defaultStyle;		/// Default application style sheet

	QList<TocChatWidget*>* 		_pChatList;		/// List of cached chat widgets
	QList<TocChatWidget*>* 		_pOpenChatList;		/// List of active/open chat widgets
	TocMainToolbar* 		_pMainToolbar;		/// Pointer to a toolbar that allows user to control the UI
	QStackedLayout* 		_pMainLayout;		///	Layout displaying all the "following" widgets
	QWidget*			_pCentralWidget;	/// Main Window's Central Widget 
	TocContactListWidget* 		_pContactList;		/// Widget displaying list of contacts 
	TocUserInfoWidget* 		_pUserInfoWidget;	/// Widget displaying contact info
	TocSettingsWidget* 		_pSettingsWidget;	/// Widget displaying settings window
	TocGeneralSettingsWidget* 	_pGeneralWidget;	/// Widget displaying general settings
	TocAccountSettingsWidget* 	_pAccountWidget;	/// Widget displaying account settings
	TocAccountIntermediateWidget* 	_pIntermediateWidget; 	/// Widget that lets user to choose account settings mode 
	TocThemesSettingsWidget* 	_pThemesWidget;		/// Widget displaying themes setting
	bool				_register;		/// Flag indicating if TietoOpCom is in "Create Account Mode"
};

QT_END_NAMESPACE

#endif // TOCMAINWINDOW_H
