/** \file TocSettings.h
 * \brief Declaration of TocSettings class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef TOCSETTINGS_H
#define TOCSETTINGS_H

#include "defs.h"

#include <QMap>
#include <QStringList>

enum SortOrder {
	Alphabetical,
	ByStatus,
	None
};

Q_DECLARE_METATYPE(SortOrder);

enum AuthorizationRule {
	Ask,
	AlwaysAccept,
	AlwaysDeny
};

Q_DECLARE_METATYPE(AuthorizationRule);

class TocSettings {

public:

	/** \brief Static public constructor
	 * 
	 * @return Pointer to the static instance
	 */
	static TocSettings* getInstance();

	/** \brief Public destructor
	 * 
	 */
	~TocSettings();

	/** \brief Getter for current account name
	 * 
	 * @return Account name
	 */
	const QString& accountName();

	/** \brief Setter for current account name
	 * 
	 * @param accountName New account name
	 */
	void setAccountName(const QString& accountName);

	/** \brief Getter for current account uid
	 * 
	 * @return Account uid
	 */
	const QString& accountUid();

	/** \brief Setter for current account uid
	 * 
	 * @param uid New account uid
	 */
	void setAccountUid(const QString& uid);

	/** \brief Getter for current account password
	 * 
	 * @return Account password
	 */
	const QString& accountPasswd();

	/** \brief Setter for current account password
	 * 
	 * @param password New account password
	 */
	void setAccountPasswd(const QString& password);

	/** \brief Getter for current protocol
	 * 
	 * @return Protocol used
	 */
	const QString& protocol();

	/** \brief Setter for current protocol
	 * 
	 * @param protocol Protocol name
	 */
	void setProtocol(const QString& protocol);

	/** \brief Getter for protocol common name
	 * 
	 * @return Protocol common used name
	 */
	const QString protocolCommonName();

	/** \brief Setter for current protocol by service name
	 * 
	 * @param service Service name
	 */
	void setProtocolByName(const QString& service);

	/** \brief Getter for available protocols
	 * 
	 * @return Available protocols
	 */
	const QStringList& protocols();

	/** \brief Setter for availabe protocols
	 * 
	 * @param protocols List of protocols names
	 */
	void setProtocols(const QStringList& protocols);

	/** \brief Getter for current server
	 * 
	 * @return Server name
	 */
	const QString& server();

	/** \brief Setter for current server
	 * 
	 * @param server Server name
	 */
	void setServer(const QString& server);

	/** \brief Getter for currently used port
	 * 
	 * @return Port number
	 */
	int port();

	/** \brief Setter for currently used port
	 * 
	 * @param port Port number
	 */
	void setPort(int port);

	/** \brief Getter for `old-ssl` connection property
	 * 
	 * @return True if connection needs `old-ssl` enabled
	 */
	bool isOldSslEnabled();

	/** \brief Setter for `old-ssl` connection property
	 * 
	 * @param bEnabled True if connection needs `old-ssl` enabled 
	 */
	void setOldSslEnabled(bool bEnabled);

	/** \brief Getter for `ignore-ssl-errors` connection property
	 * 
	 * @return True if connection needs `ignore-ssl-errors` enabled
	 */
	bool isIgnoreSslErrorsEnabled();

	/** \brief Setter for `ignore-ssl-errors` connection property
	 * 
	 * @param bEnabled True if connection needs `ignore-ssl-errors` enabled 
	 */
	void setIgnoreSslErrorsEnabled(bool bEnabled);

	/** \brief Getter for saved contact list
	 * 
	 * @return List of TocContact's
	 */
	TocContactList* contactList();
	
	/** \brief Getter for list of contacts to be added to server
	 * 
	 * @return List of Uids to be added
	 */
	QStringList* contactsToAdd();

	/** \brief Getter for list of contacts to be removed from server
	 * 
	 * @return List of Uids to be removed
	 */
	QStringList* contactsToRemove();

	/** \brief Getter for sort order policy
	 * 
	 * @return Sorting order policy
	 */
	SortOrder sortOrder();

	/** \brief Setter for sort order policy
	 * 
	 * @param sortOrder Sorting order policy
	 */
	void setSortOrder(SortOrder sortOrder);

	/** \brief Getter for authorization rule
	 * 
	 * @return Authorization rule
	 */
	AuthorizationRule authorizationRule();

	/** \brief Setter for authorization rule
	 * 
	 * @param rule Authorization rule
	 */
	void setAuthorizationRule(AuthorizationRule rule);

	/** \brief Getter for current ui theme
	 * 
	 * @return File name of the style sheet file
	 */
	const QString& currentTheme();

	/** \brief Setter for current ui theme
	 * 
	 * @param theme File name of the style sheet file
	 */
	void setCurrentTheme(const QString& theme);

	/** \brief Returns a server hint for given data
	 * 
	 * @param uid Account login
	 * @param serviceName Common name of the service/protocol (eg. Jabber, Google Talk, SIP)
	 * @return Default server for the data
	 */
	const QString& serverHint( const QString& uid, const QString& serviceName);

	/** \brief Returns a port hint for given data
	 * 
	 * @param uid Account login
	 * @param serviceName Common name of the service/protocol (eg. Jabber, Google Talk, SIP)
	 * @return Default server for the data
	 */
	int portHint( const QString& uid, const QString& serviceName);

	/** \brief Returns true if such protocol is already supported
	 * 
	 * @param protocol Protocol to check
	 * @return True if protocol already exist
	 */
	bool isProtocolInstalled(const QString& protocol);

	/** \brief Add a new protocol to the list of supported protocols
	 * 
	 * @param commonName	Protocol's common name
	 * @param protocol		Protocol's identifier (according to the Telepathy spec)
	 */
	void installProtocol(const QString& commonName, const QString& protocol);

	/** \brief One shot getter for the `register` parameter of the connection
	 * 
	 * When it returns the "true" value it sets it immediately to "false".
	 * 
	 * @return true if user wants to register an account instead of logging in
	 */
	bool isRegister();

	/** \brief Setter for the `register` parameter of the connection
	 * 
	 * When it is invoked it sets the `register` parameter to "true"
	 *
	 */
	void setRegister();

private:
	
	/** \brief Private constructor
	 * 
	 */
	TocSettings();

	/** \brief Used to read settings with QSettings
	 * 
	 *  It's meant to be used only in destructor. That's why it's private. 
	 */
	void read();
	
	
	/** \brief Used to save settings with QSettings
	 * 
	 *  It's meant to be used only when creating an instance. That's why it's private. 
	 */
	void save();	
	
	static TocSettings*		_pSelf;

	QString					_accountName;
	QString					_accountUid;
	QString					_accountPasswd;
	QString					_protocol;
	QStringList				_protocols;
	QString					_protocolCommonName;
	QString					_server;
	QString					_serverHint;
	int						_port;
	int						_portHint;
	bool					_bOldSsl;
	bool					_bIgnoreSslErrors;
	QList<TocContact*>*		_pContactList;
	QStringList				_addedContacts;
	QStringList				_removedContacts;
	AuthorizationRule		_authorizationRule;
	SortOrder				_sortOrder;
	QString					_currentTheme;
	QMap<QString, QString>*	_pProtocolMap;
	bool					_register;
};

#endif  // TOCSETTINGS_H
