/** \file TocEngine.cpp
 * \brief Implementation of TocEngine class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "TocEngine"
#include "TocSettings"
#include "macros.h"

#include <QString>
#include <QDebug>

#include <QtTapioca/ConnectionManager>
#include <QtTapioca/ConnectionManagerFactory>
#include <QtTapioca/Contact>
#include <QtTapioca/ContactList>
#include <QtTapioca/TextChannel>

using namespace QtTapioca;

Q_DECLARE_METATYPE(QtTapioca::Connection*);
Q_DECLARE_METATYPE(QtTapioca::ContactBase*);
Q_DECLARE_METATYPE(QtTapioca::Connection::Status);
Q_DECLARE_METATYPE(QtTapioca::Connection::Reason);
Q_DECLARE_METATYPE(QtTapioca::ContactBase::Presence);

TocEngine::TocEngine( QObject * parent ): 
	TocBaseEngine( parent ), 
	_bNewUserContact(true), 
	_bContactListSynchronized(false), 
	_pConnection(0) 
{

	qRegisterMetaType<QtTapioca::Connection::Status>();
	qRegisterMetaType<QtTapioca::Connection::Reason>();
	qRegisterMetaType<QtTapioca::ContactBase::Presence>();
	qRegisterMetaType<TocContactList>();

	TocSettings* pSettings = TocSettings::getInstance();
	_pTocContactList = pSettings->contactList();
	_pAddedContacts = pSettings->contactsToAdd();
	_pRemovedContacts = pSettings->contactsToRemove();
	
}

TocEngine::~TocEngine() {
	if(_pConnection)
		_pConnection->disconnect();

	delete _pConnection;
	
	TocSettings *pSettings = TocSettings::getInstance();
	delete pSettings;
}

void TocEngine::initializeFromSettings() {

	_bNewUserContact = true;

	TocSettings* pSettings = TocSettings::getInstance();

	// Disconnect first
	if(_pConnection) {
		_pConnection->disconnect();
		delete _pConnection;
		_pConnection = 0;
	}
	// Reset the UserContact flag
	_bNewUserContact = true;
	
	ConnectionManagerFactory *pFactory = ConnectionManagerFactory::self();

	// Find new available protocols
	QList<ConnectionManager*> connectionManagers = pFactory->getAllConnectionManagers();
	ConnectionManager *pConnectionManager;
	foreach ( pConnectionManager, connectionManagers ) {
		QString protocol;
		QStringList protocols = pConnectionManager->supportedProtocols();
		foreach (protocol, protocols) {
			if( !pSettings->isProtocolInstalled( protocol ) )
				pSettings->installProtocol( pConnectionManager->name() + "/" + protocol, protocol );
		}
	}

	// Get Connection Manager
	pConnectionManager = pFactory->getConnectionManager( pSettings->protocol() );
	if (!pConnectionManager) {
		_pConnection = 0;
		return;
	}
	
	// Search for "lost" connection
	QList<Connection *> connectionsList = pConnectionManager->connections();
	if( !connectionsList.empty() ) {
		Connection *pConnection;
		QString account = static_cast<QString>(pSettings->protocol() ) + "." +
				static_cast<QString>(pSettings->accountUid()).replace("@", "_40").replace(".", "_2e");
		foreach(pConnection, connectionsList) {
			if( pConnection->serviceName().contains(account) ) {

				// This doesn't always work... why?
				pConnection->disconnect();

				_pConnection = pConnection;
				CONNECT(_pConnection, SIGNAL(statusChanged(QtTapioca::Connection*,QtTapioca::Connection::Status,QtTapioca::Connection::Reason)), this, SLOT(onStatusChange(QtTapioca::Connection*,QtTapioca::Connection::Status,QtTapioca::Connection::Reason)));
				CONNECT(_pConnection, SIGNAL(channelCreated(QtTapioca::Connection*,QtTapioca::Channel*,bool)),
						this, SLOT(onChannelCreated(QtTapioca::Connection*,QtTapioca::Channel*,bool)));
			
				return;
			}
		}
	}

	// Setup connection parameters
	QList<ConnectionManager::Parameter> parameters;
	parameters.append(ConnectionManager::Parameter("account", pSettings->accountUid()));
	parameters.append(ConnectionManager::Parameter("password", pSettings->accountPasswd()));
	if( !pSettings->server().isEmpty() )
		parameters.append(ConnectionManager::Parameter("server", pSettings->server()));
	if( pSettings->port() > 0 )
		parameters.append(ConnectionManager::Parameter("port", QVariant( (uint)pSettings->port() )));
	if( pSettings->isOldSslEnabled() )
		parameters.append(ConnectionManager::Parameter("old-ssl", QVariant(true)));
	if( pSettings->isIgnoreSslErrorsEnabled() )
		parameters.append(ConnectionManager::Parameter("ignore-ssl-errors", QVariant(true)));
	if( pSettings->isRegister() )
		parameters.append(ConnectionManager::Parameter("register", QVariant(true)));

	_pConnection = pConnectionManager->requestConnection( pSettings->protocol(), parameters );
	
	if( !_pConnection ) {
		return;
	}

	CONNECT(_pConnection, SIGNAL(statusChanged(QtTapioca::Connection*,QtTapioca::Connection::Status,QtTapioca::Connection::Reason)), 
		this, SLOT(onStatusChange(QtTapioca::Connection*,QtTapioca::Connection::Status,QtTapioca::Connection::Reason)));	
	CONNECT(_pConnection, SIGNAL(channelCreated(QtTapioca::Connection*,QtTapioca::Channel*,bool)),
		this, SLOT(onChannelCreated(QtTapioca::Connection*,QtTapioca::Channel*,bool)));
}

void TocEngine::onPresenceChange(Presence presence, QString description) {
	
	if( !_pConnection )
		initializeFromSettings();

	if( !_pConnection )
		return;

	if( _pConnection->status() == QtTapioca::Connection::Connected && _pConnection->userContact())
		_pConnection->userContact()->setPresenceWithMessage(static_cast <QtTapioca::ContactBase::Presence> (presence), static_cast <QString> (description));
	
	else if( _pConnection->status() == QtTapioca::Connection::Disconnected )
		if(presence == Offline)
				_bContactListSynchronized = false; // Just to be sure
		else
			_pConnection->connect(static_cast <QtTapioca::ContactBase::Presence> (presence), static_cast <QString> (description));
}

void TocEngine::onStatusChange(QtTapioca::Connection* pConnection, QtTapioca::Connection::Status status, QtTapioca::Connection::Reason reason) {
	
	Q_UNUSED(pConnection);
	
	emit statusChanged(static_cast <Status> (status), static_cast <Reason> (reason));
		
	if( _bNewUserContact && _pConnection->userContact() && _pConnection->contactList() && _pConnection->status() == QtTapioca::Connection::Connected) {

		CONNECT(_pConnection->userContact(), SIGNAL(presenceUpdated(QtTapioca::ContactBase*,QtTapioca::ContactBase::Presence,QString)), this, SLOT(onSelfPresenceUpdated(QtTapioca::ContactBase*,QtTapioca::ContactBase::Presence,QString)));

		// disconnect for safety of double connections because we don't really know if it's the same object or not..
		disconnect(_pConnection->contactList(), SIGNAL(authorizationRequested(QtTapioca::Contact*)), this, SLOT(onAuthorizationRequested(QtTapioca::Contact*)));
		
		CONNECT(_pConnection->contactList(), SIGNAL(authorizationRequested(QtTapioca::Contact*)), this, SLOT(onAuthorizationRequested(QtTapioca::Contact*)));
		_bNewUserContact = false;
		
	} else if(_pConnection->status() == QtTapioca::Connection::Disconnected) {
		
		_bContactListSynchronized = false;
//		delete _pConnection; // TODO: Why does it crash??
		_pConnection = 0;
	}
}

void TocEngine::onPresenceUpdated(QtTapioca::ContactBase* pContact, QtTapioca::ContactBase::Presence presence, const QString &presenceMessage) {

	emit contactPresenceUpdated(pContact->uri(), static_cast <Presence> (presence), presenceMessage);
}

void TocEngine::onSelfPresenceUpdated(QtTapioca::ContactBase* pContact, QtTapioca::ContactBase::Presence presence, const QString &presenceMessage) {

	Q_UNUSED(pContact);

	emit presenceUpdated(static_cast <Presence> (presence), presenceMessage);

	if(!_bContactListSynchronized) {
		synchronizeContactLists();
		_bContactListSynchronized = true;
		
	} else if(presence == QtTapioca::ContactBase::Offline) {
			_bContactListSynchronized = false;
			_pConnection->disconnect();
	}
}

void TocEngine::onAddContactRequest(TocContact* pContact){

	_pTocContactList->append(pContact);
	_pAddedContacts->append( pContact->uid );
	_pRemovedContacts->removeOne( pContact->uid );

	addPendingContacts();
}

void TocEngine::addPendingContacts() {

	if( _pConnection && _pConnection->status() == Connection::Connected ) {

		while( !_pAddedContacts->isEmpty() ) {
			QString uid = _pAddedContacts->takeFirst();
			Contact* pContact = _pConnection->contactList()->addContact( uid );
			if( pContact ) { // If it suceeded
				pContact->authorize(true); //TODO: This should be removed when we have the facility to grant authorization at a later time
				pContact->subscribe(true); // Ask for subscription
				
				CONNECT(pContact, SIGNAL(presenceUpdated(QtTapioca::ContactBase*,QtTapioca::ContactBase::Presence,const QString&)),
						this, SLOT(onPresenceUpdated(QtTapioca::ContactBase*, QtTapioca::ContactBase::Presence, const QString&)));
			} else {}
				// TODO: Notify UI about the error (usually UID not valid for the protocol)
		}
	}
}


void TocEngine::onEditContactRequest(TocContact* pContact, QString oldUid){

	if( pContact->uid != oldUid ) {
		
		if(!_pAddedContacts->removeOne( oldUid ))
			_pRemovedContacts->append( oldUid );
		
		_pAddedContacts->append( pContact->uid );
		_pRemovedContacts->removeOne( pContact->uid );
		removePendingContacts();
		addPendingContacts();
	}
}

void TocEngine::onRemoveContactRequest(QString uid) {

	if(!_pTocContactList) return;
	
	for(int i = 0; i < _pTocContactList->count(); ++i)
		if( _pTocContactList->at(i)->uid == uid ) {
			_pTocContactList->removeAt(i);
			//TODO: remove cached picture if needed
		}

	if(!_pAddedContacts->removeOne( uid ))
		_pRemovedContacts->append( uid );

	removePendingContacts();
}

void TocEngine::removePendingContacts() {

	if( _pConnection && _pConnection->status() == Connection::Connected ) {

		while( !_pRemovedContacts->isEmpty() ) {
			QString uid = _pRemovedContacts->takeFirst();
			Contact* pContact = _pConnection->contactList()->contact( uid );
			if( pContact )
				_pConnection->contactList()->removeContact( pContact );
		}
	}
}

int TocEngine::indexForUid( QString uid ) {

	for(int i = 0; i < _pTocContactList->count(); ++i)
		if( _pTocContactList->at(i)->uid == uid ) {
			return i;
		}
	
	return -1;
}

void TocEngine::removeContact( QString uid ) {
	
	int i = indexForUid(uid);
	if( i != -1 ) {
		_pTocContactList->removeAt(i);
		//TODO: remove cached picture if needed
	}
}

void TocEngine::addContact( const Contact* pContact ) {
	
	TocContact* pTocContact = new TocContact;
	pTocContact->uid = pContact->uri();
	pTocContact->name = pContact->alias(); // That's TOO BAD! I cann't set it to MY OWN selected name.. because of lack of support by telepathy
	pTocContact->customIconId = 0;
	pTocContact->customIconPath = "";
	pTocContact->gender = NotSpecified;
	pTocContact->presence = Offline;

	_pTocContactList->append(pTocContact);
}

void TocEngine::syncContactsFromServer() {

	QStringList localContactsList;

	QList<Contact*> serverContactsList = _pConnection->contactList()->knownContacts();

	for(int i = 0; i < _pTocContactList->count(); ++i)
		localContactsList.append( _pTocContactList->at(i)->uid );

	// Find differences
	int idx = 0;
	while( idx < serverContactsList.count() ) {

		int idy = localContactsList.indexOf( serverContactsList.at(idx)->uri() );

		if( idy != -1 ) {
			serverContactsList.removeAt(idx);
			localContactsList.removeAt(idy);
			--idx;
		}

		++idx;
	}
	
	// Now we're left with two lists,
	// one with contacts to be removed 
	// and one to be added to our local list.

	// To add
	Contact* pContact;	
	foreach(pContact, serverContactsList) {

		addContact(pContact);
		
		CONNECT(pContact, SIGNAL(presenceUpdated(QtTapioca::ContactBase*,QtTapioca::ContactBase::Presence,const QString&)),
				this, SLOT(onPresenceUpdated(QtTapioca::ContactBase*, QtTapioca::ContactBase::Presence, const QString&)));
						
	}

	// To remove //TODO: Maybe add an option to turn off removal of contacts that are removed on server by other clients
	QString uid;	
	
	foreach(uid, localContactsList) {

		removeContact(uid);
	}
}

void TocEngine::synchronizeContactLists() {
	
	Contact* pContact;
	foreach(pContact, _pConnection->contactList()->knownContacts())
		CONNECT(pContact, SIGNAL(presenceUpdated(QtTapioca::ContactBase*,QtTapioca::ContactBase::Presence,const QString&)),
				this, SLOT(onPresenceUpdated(QtTapioca::ContactBase*, QtTapioca::ContactBase::Presence, const QString&)));

	addPendingContacts();
	removePendingContacts();
	syncContactsFromServer();

	emit contactListReceived(*_pTocContactList);
	_bContactListSynchronized = true;

}

void TocEngine::onNewMessageReady(QString uid, Message message) {
	
	if( !_pConnection || _pConnection->status() != QtTapioca::Connection::Connected) {
		message.error = UserOffline;
		emit incomingMessage(uid, message);
		return; // Message that You cann't send messages when offline
	}

	Channel* pDestinationChannel = NULL;
	bool bNewChannel = false;
	
	QList<Channel*> pChannels = _pConnection->openChannels();
	int i;
	for(i = 0; i < pChannels.count(); ++i)
		if(pChannels[i]->target()->uri() == uid)
			break;

	if(i == pChannels.count()) { //not found, create new channel
		Contact* pContact = _pConnection->contactList()->contact(uid);
		if(!pContact) {
			message.error = NotOnContactList;
			emit incomingMessage(uid, message);
			return; // Message that You cann't send messages to the contact not on users contact list
		}
		pDestinationChannel = _pConnection->createChannel(Channel::Text, pContact);
		if(pDestinationChannel)
			bNewChannel = true;
		else { // Channel could not be created
			message.error = CannotCreateChannel;
			emit incomingMessage(uid, message);
			return; // Message that Channel could not be created
		}
	} else {
		pDestinationChannel = pChannels[i];
	}
	
	if(pDestinationChannel->type() == Channel::Text) {
		
		TextChannel* pTextChannel = dynamic_cast<TextChannel*>(pDestinationChannel);
		
		if(bNewChannel) {
			
			CONNECT(pTextChannel, SIGNAL(messageReceived(const QtTapioca::TextChannel*,const QtTapioca::TextChannel::Message&)),
					this, SLOT(onMessageReceived(const QtTapioca::TextChannel*,const QtTapioca::TextChannel::Message&)));
			CONNECT(pTextChannel, SIGNAL(messageDeliveryError(const QtTapioca::TextChannel::Message&, QtTapioca::TextChannel::Message::DeliveryError)),
					this, SLOT(onMessageDeliveryError(const QtTapioca::TextChannel::Message&, QtTapioca::TextChannel::Message::DeliveryError)));
		}
		
		QtTapioca::TextChannel::Message mess(message.contents);
		pTextChannel->sendMessage(mess);
	}
}

void TocEngine::onMessageReceived(const QtTapioca::TextChannel* pTextChannel, const QtTapioca::TextChannel::Message& message) {

	struct Message tecMessage;
	tecMessage.contents = message.contents();
	tecMessage.timestamp = message.timestamp();
	tecMessage.error = NoError;
	
	if(pTextChannel) {
		emit incomingMessage(pTextChannel->target()->uri(), tecMessage);
		const_cast<QtTapioca::TextChannel*>(pTextChannel)->acknowledge(message);
	}
}

void TocEngine::onMessageDeliveryError(const QtTapioca::TextChannel::Message &message, QtTapioca::TextChannel::Message::DeliveryError error) {
	
	struct Message tecMessage;
	tecMessage.contents = message.contents();
	tecMessage.timestamp = message.timestamp();
	tecMessage.error = static_cast<MessageDeliveryError>(error);
	
	QObject* pSender = sender();
	if(pSender) {
		TextChannel* pTextChannel = dynamic_cast<TextChannel*>(pSender);
		emit incomingMessage(pTextChannel->target()->uri(), tecMessage);
	}
}

void TocEngine::onChannelCreated(QtTapioca::Connection* pConnection, QtTapioca::Channel* pChannel, bool bSuppressHandler) {
	
	Q_UNUSED(pConnection);
	Q_UNUSED(bSuppressHandler);

	if(pChannel->type() == Channel::Text) {
		TextChannel* pTextChannel = dynamic_cast<TextChannel*>(pChannel);
		CONNECT(pTextChannel, SIGNAL(messageReceived (const QtTapioca::TextChannel*,const QtTapioca::TextChannel::Message&)),
				this, SLOT(onMessageReceived(const QtTapioca::TextChannel*,const QtTapioca::TextChannel::Message&)));
		CONNECT(pTextChannel, SIGNAL(messageDeliveryError(const QtTapioca::TextChannel::Message&, QtTapioca::TextChannel::Message::DeliveryError)),
							this, SLOT(onMessageDeliveryError(const QtTapioca::TextChannel::Message&, QtTapioca::TextChannel::Message::DeliveryError)));

		QList<TextChannel::Message> messages = pTextChannel->pendingMessages();
		if(messages.count()) {
			int i;
			struct Message message;
			message.error = NoError;
			for(i = 0; i < messages.count(); ++i) {
				message.contents = messages.at(i).contents();
				message.timestamp = messages.at(i).timestamp();
				emit incomingMessage(pChannel->target()->uri(), message);
			}
		}
	}
}

void TocEngine::onSessionClosed(QString uid) {
	
	if( !_pConnection )
		return;

	QList<Channel*> pChannels = _pConnection->openChannels();
	int i;
	for(i = 0; i < pChannels.count(); ++i)
		if(pChannels[i]->target()->uri() == uid)
			break;

	if(i < pChannels.count()) {
		pChannels.at(i)->close();
	}
}

void TocEngine::onAuthorizationRequested(QtTapioca::Contact *pContact) {
	
	// TODO: should be dependent on user pSettings
	pContact->authorize( true );
	pContact->subscribe( true );

	for(int i = 0; i < _pTocContactList->count(); ++i)
		if(_pTocContactList->at(i)->uid == pContact->uri())
			return; // Return if the contact is already on our contact list

	addContact(pContact); // Add Contact to notify user
	CONNECT(pContact, SIGNAL(presenceUpdated(QtTapioca::ContactBase*,QtTapioca::ContactBase::Presence,const QString&)),
		this, SLOT(onPresenceUpdated(QtTapioca::ContactBase*, QtTapioca::ContactBase::Presence, const QString&)));

	emit contactListReceived(*_pTocContactList);
}
