/** \file TocSettings.cpp
 * \brief Implementation of TocSettings class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "TocSettings"

#include <QSettings>
#include <QDebug>

// Initialize static objects
TocSettings* TocSettings::_pSelf = 0;

TocSettings* TocSettings::getInstance() {

	if (!_pSelf)
		_pSelf = new TocSettings();

	return _pSelf;
}

TocSettings::TocSettings() 
	: _register(false) {
		
	_pProtocolMap = new QMap<QString, QString>();
	_pProtocolMap->insert("Jabber", "jabber");
	_pProtocolMap->insert("SIP", "sip");
	_pProtocolMap->insert("Google Talk", "jabber");

	_pContactList = new TocContactList();

	read();
}

TocSettings::~TocSettings() {

	save();
	
	delete _pProtocolMap;
	delete _pContactList;
}

void TocSettings::read() {
	
	QSettings settings("Tieto", "tietoopcom");
	settings.beginGroup("Account");
	_accountName 		= settings.value("account/name").toString();
	_accountUid 		= settings.value("account/uid").toString();
	_accountPasswd 	 	= settings.value("account/pass").toString();
	_protocol 			= settings.value("server/protocol").toString();
	_protocols 		 	= _pProtocolMap->keys();
	_protocolCommonName = settings.value("server/protocol-name").toString();
	_server 			= settings.value("server/server").toString();
	_serverHint		 	= "";
	_port 				= settings.value("server/port").toInt();
	_portHint			= 0;
	_bOldSsl 			= settings.value("server/old-ssl", false).toBool();
	_bIgnoreSslErrors 	= settings.value("server/ignore-ssl-errors", false).toBool();
	
	settings.endGroup();
	
	settings.beginGroup("Contact_List");
	_authorizationRule	= static_cast<AuthorizationRule>(settings.value("list/authorization-rule", Ask).toInt());
	_sortOrder 		= static_cast<SortOrder>(settings.value("list/sort-order", Alphabetical).toInt());

	if(!_pContactList)
		_pContactList = new TocContactList();
	else
		_pContactList->clear();
	
	settings.beginGroup("Contacts");
	
	QStringList contactsList = settings.childKeys();
	
	foreach(QString entry, contactsList) {
		
		QStringList contactFields = settings.value(entry).toString().split("|");
		
		TocContact* pContact = new TocContact();
		pContact->uid = entry;
		pContact->name = contactFields[0];
		pContact->customIconPath = contactFields[1];
		pContact->customIconId = 0;
		pContact->gender = static_cast<Gender>(contactFields[2].toInt());
		pContact->presence = Offline;
		
		_pContactList->append(pContact);
	}
	
	
	settings.endGroup();
	settings.endGroup();

	settings.beginGroup("Pending_Contact_Lists");

	settings.beginGroup("Added");
	_addedContacts = settings.childKeys();
	
	settings.endGroup();

	settings.beginGroup("Removed");
	_removedContacts = settings.childKeys();
	
	settings.endGroup();
	settings.endGroup();
	
	settings.beginGroup("Themes");
	
#ifdef QT_NO_DEBUG
  #define ssPath    	QString("/usr/share/tietoopcom/stylesheets/")
#else
  #define ssPath    	QString("./data/stylesheets/")
#endif //QT_NO_DEBUG
	
	_currentTheme = settings.value("theme", ssPath + "/default.qss").toString();
	settings.endGroup();
}

void TocSettings::save() {

	QSettings settings("Tieto", "tietoopcom");
		
	settings.beginGroup("Account");
	settings.setValue("account/name", _accountName);
	settings.setValue("account/uid", _accountUid);
	settings.setValue("account/pass", _accountPasswd);
	settings.setValue("server/protocol", _protocol);
	settings.setValue("server/protocol-name", _protocolCommonName);
	settings.setValue("server/server", _server);
	settings.setValue("server/port", _port);
	settings.setValue("server/old-ssl", _bOldSsl);
	settings.setValue("server/ignore-ssl-errors", _bIgnoreSslErrors);
	settings.endGroup();
	
	settings.beginGroup("Contact_List");
	settings.setValue("list/authorization-rule", _authorizationRule);
	settings.setValue("list/sort-order", _sortOrder);
	
		
	settings.beginGroup("Contacts");
	settings.remove("");
	
	QString entry;
	TocContact* pContact = 0;

	foreach(pContact, *_pContactList) {

		entry =  pContact->name + "|" +  pContact->customIconPath + "|" +  QString::number(static_cast<int>(pContact->gender)); 
		settings.setValue(pContact->uid, entry);
	}

	settings.endGroup();
	settings.endGroup();

	settings.beginGroup("Pending_Contact_Lists");
	settings.beginGroup("Added");
	settings.remove("");
	
	foreach(QString added, _addedContacts)
		settings.setValue(added, 0);
	
	settings.endGroup ();

	settings.beginGroup("Removed");
	settings.remove("");
	
	foreach(QString removed, _removedContacts)
		settings.setValue(removed, 0);
	
	settings.endGroup();
	settings.endGroup();
	
	settings.beginGroup("Themes");
	settings.setValue("theme", _currentTheme);
	settings.endGroup();
}

const QString& TocSettings::accountName() {
	return _accountName;
}

void TocSettings::setAccountName(const QString& accountName) {
	_accountName = accountName;
}

const QString& TocSettings::accountUid() {
	return _accountUid;
}

void TocSettings::setAccountUid(const QString& uid) {
	_accountUid = uid;
}

const QString& TocSettings::accountPasswd() {
	return _accountPasswd;
}

void TocSettings::setAccountPasswd(const QString& password) {
	_accountPasswd = password;
}

const QString& TocSettings::protocol() {
	return _protocol;
}

void TocSettings::setProtocol(const QString& protocol) {
	_protocol = protocol;
}

void TocSettings::setProtocolByName(const QString& service) {
	
	_protocol = _pProtocolMap->value(service);
	_protocolCommonName = service;

	if(service == "Jabber") {
		_bOldSsl 			= false;
		_bIgnoreSslErrors	= false;
	} else if( service == "Google Talk") {
		_bOldSsl 			= true;
		_bIgnoreSslErrors	= true;
	} else if( service == "SIP") {
		_bOldSsl 			= false;
		_bIgnoreSslErrors	= false;
	} else {
		_bOldSsl 			= false;
		_bIgnoreSslErrors	= false;
	}
}

const QStringList& TocSettings::protocols() {
	return _protocols;
}

void TocSettings::setProtocols(const QStringList& protocols) {
	_protocols = protocols;
}

const QString& TocSettings::server() {
	return _server;
}

void TocSettings::setServer(const QString& server) {
	_server = server;
}

int TocSettings::port() {
	return _port;
}

void TocSettings::setPort(int port) {
	_port = port;
}

bool TocSettings::isOldSslEnabled() {
	return _bOldSsl;
}

void TocSettings::setOldSslEnabled(bool bEnabled) {
	_bOldSsl = bEnabled;
}

bool TocSettings::isIgnoreSslErrorsEnabled() {
	return _bIgnoreSslErrors;
}

void TocSettings::setIgnoreSslErrorsEnabled(bool bEnabled) {
	_bIgnoreSslErrors = bEnabled;
}

QList<TocContact*>* TocSettings::contactList() {
	return _pContactList;
}

QStringList* TocSettings::contactsToAdd() {
	return &_addedContacts;
}

QStringList* TocSettings::contactsToRemove() {
	return &_removedContacts;
}

SortOrder TocSettings::sortOrder() {
	return _sortOrder;
}

void TocSettings::setSortOrder(SortOrder sortOrder) {
	_sortOrder = sortOrder;
}

AuthorizationRule TocSettings::authorizationRule() {
	return _authorizationRule;
}

void TocSettings::setAuthorizationRule(AuthorizationRule rule) {
	_authorizationRule = rule;
}

const QString& TocSettings::currentTheme() {
	return _currentTheme;
}

void TocSettings::setCurrentTheme(const QString& theme) {
	_currentTheme = theme;
}

bool TocSettings::isProtocolInstalled(const QString& protocol) {
	return !_pProtocolMap->key(protocol).isEmpty();
}

void TocSettings::installProtocol(const QString& commonName, const QString& protocol) {
	_pProtocolMap->insert(commonName, protocol);
	_protocols = _pProtocolMap->keys();
}

const QString TocSettings::protocolCommonName() {
	return _protocolCommonName;
}

int TocSettings::portHint(const QString& uid, const QString& serviceName) {
	
	Q_UNUSED(uid);
	
	if( serviceName == "Jabber")
		return _portHint = 5222;
	if( serviceName == "Google Talk")
		return _portHint = 5223;
	if( serviceName == "SIP")
		return _portHint = 5060;

	return _portHint = 0;
}

const QString& TocSettings::serverHint(const QString& uid, const QString& serviceName) {
	
	if(serviceName == "Jabber")
		if(uid.contains("@"))
			return _serverHint = uid.split("@").at(1);
	if(serviceName == "Google Talk")
		return _serverHint = "talk.google.com";
	if(serviceName == "SIP")
		return _serverHint = "";

	return _serverHint = "";
}

bool TocSettings::isRegister() {
	bool bRet = _register;
	_register = false;
	return bRet;
}

void TocSettings::setRegister() {
	_register = true;
}
