/** \file IconProvider.cpp
 * \brief Implementation of IconProvider class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include <QPainter>
#include <QIcon>
#include <QDebug>

#include "IconProvider"
#include "TocSettings"


// Initialize static object
IconProvider* IconProvider::mp_self = 0;

// ---------------------------------------------------------------------------
// getInstance
// ---------------------------------------------------------------------------
//
IconProvider* IconProvider::getInstance() 
{
	if ( !mp_self)
		mp_self = new IconProvider();

	return mp_self;
}

// ---------------------------------------------------------------------------
// Constructor
// ---------------------------------------------------------------------------
//
IconProvider::IconProvider():
	_customIconId(1),
	_defaultMaleImagePath(":/contactmale.png"),
	_defaultFemaleImagePath(":/contactfemale.png"),
	_defaultUnknownImagePath(":/unknown.png")
{
	//Initialize _defaultIcons map according to gender and status
	
	//Male
	QImage* pNoFrame = new QImage(_defaultMaleImagePath);
	QImage* pRed 	= addStatusFrame(pNoFrame, Busy);
	QImage* pYellow = addStatusFrame(pNoFrame, Away);
	QImage* pGreen 	= addStatusFrame(pNoFrame, Available);
	
	_defaultIcons[Male << 8 | Hidden] 	= pNoFrame;
	_defaultIcons[Male << 8 | Offline] 	= pNoFrame;
	_defaultIcons[Male << 8 | Available]= pGreen;
	_defaultIcons[Male << 8 | Busy]		= pRed;
	_defaultIcons[Male << 8 | XA] 	 	= pYellow;
	_defaultIcons[Male << 8 | Away] 	= pYellow;

	//Female
	pNoFrame = new QImage(_defaultFemaleImagePath);
	pRed 	= addStatusFrame(pNoFrame, Busy);
	pYellow = addStatusFrame(pNoFrame, Away);
	pGreen 	= addStatusFrame(pNoFrame, Available);
	
	_defaultIcons[Female << 8 | Hidden] 	= pNoFrame;
	_defaultIcons[Female << 8 | Offline] 	= pNoFrame;
	_defaultIcons[Female << 8 | Available]	= pGreen;
	_defaultIcons[Female << 8 | Busy]		= pRed;
	_defaultIcons[Female << 8 | XA] 	 	= pYellow;
	_defaultIcons[Female << 8 | Away] 	 	= pYellow;

	//NotSpecified
	pNoFrame = new QImage(_defaultUnknownImagePath);
	pRed 	= addStatusFrame(pNoFrame, Busy);
	pYellow = addStatusFrame(pNoFrame, Away);
	pGreen 	= addStatusFrame(pNoFrame, Available);
	
	_defaultIcons[NotSpecified << 8 | Hidden] 	= pNoFrame;
	_defaultIcons[NotSpecified << 8 | Offline] 	= pNoFrame;
	_defaultIcons[NotSpecified << 8 | Available]= pGreen;
	_defaultIcons[NotSpecified << 8 | Busy]		= pRed;
	_defaultIcons[NotSpecified << 8 | XA] 	 	= pYellow;
	_defaultIcons[NotSpecified << 8 | Away] 	= pYellow;
		

	//Initialize _customIcons map from TocSettings
	TocSettings *pSettings = TocSettings::getInstance();
	
	TocContactList *pContacts = pSettings->contactList();
	
	foreach(TocContact* pContact, *pContacts)
	{
		if(!pContact->customIconPath.isEmpty())
			setCustomIcon(pContact);
	}
}

// ---------------------------------------------------------------------------
// Destructor
// ---------------------------------------------------------------------------
//
IconProvider::~IconProvider()
{
	QList<int> defaultIconsKeys = _defaultIcons.uniqueKeys();
	
	foreach(int key, defaultIconsKeys)
	{
		QImage* pImage = _defaultIcons.value(key);
		if(pImage)
		{
			delete pImage;
		}
	}
	
	
	QList<int> customIconsKeys = _defaultIcons.uniqueKeys();
	
	foreach(int key, customIconsKeys)
	{
		QImage* pImage = _customIcons.value(key);
		if(pImage)
		{
			delete pImage;
		}
	}	
}

// ---------------------------------------------------------------------------
// matchIcon
// ---------------------------------------------------------------------------
//
QImage* IconProvider::matchIcon(const TocContact* pContact) const
{
	if(pContact->customIconId)
		return _customIcons.value(pContact->customIconId << 8 | pContact->presence);
	else
		return _defaultIcons.value(pContact->gender << 8 | pContact->presence);
}

// ---------------------------------------------------------------------------
// matchIcon
// ---------------------------------------------------------------------------
//
QIcon IconProvider::matchIcon(Gender gender) const
{
	switch( static_cast<int>(gender) ) 
	{

		case NotSpecified:
			return QIcon(_defaultUnknownImagePath);

		case Male:
			return QIcon(_defaultMaleImagePath);		

		case Female:
			return QIcon(_defaultFemaleImagePath);		

		default:
			return QIcon(_defaultUnknownImagePath);
	}
}

// ---------------------------------------------------------------------------
// setCustomIcon
// ---------------------------------------------------------------------------
//
void IconProvider::setCustomIcon(TocContact* pContact)
{
	//Check if there's something to be set
	if(pContact->customIconPath.isEmpty())
	
	//Check if this pContact has some custom icon already associated 
	if(_customIcons[pContact->customIconId])
	{
		//if yes, remove it from the list
		_customIcons.remove(pContact->customIconId << 8 | Hidden);
		_customIcons.remove(pContact->customIconId << 8 | Offline);
		_customIcons.remove(pContact->customIconId << 8 | Available);
		_customIcons.remove(pContact->customIconId << 8 | Busy);
		_customIcons.remove(pContact->customIconId << 8 | XA);
		_customIcons.remove(pContact->customIconId << 8 | Away);
	}
	
	QImage* pNoFrame = new QImage(pContact->customIconPath);
	QImage* pRed 	= addStatusFrame(pNoFrame, Busy);
	QImage* pYellow = addStatusFrame(pNoFrame, Away);
	QImage* pGreen 	= addStatusFrame(pNoFrame, Available);
		
	_customIcons[_customIconId << 8 | Hidden] 		= pNoFrame;
	_customIcons[_customIconId << 8 | Offline] 		= pNoFrame;
	_customIcons[_customIconId << 8 | Available]	= pGreen;
	_customIcons[_customIconId << 8 | Busy]			= pRed;
	_customIcons[_customIconId << 8 | XA] 	 		= pYellow;
	_customIcons[_customIconId << 8 | Away] 		= pYellow;
	
	pContact->customIconId = _customIconId++;
}

// ---------------------------------------------------------------------------
// addStatusFrame
// ---------------------------------------------------------------------------
//
QImage* IconProvider::addStatusFrame(const QImage *baseImage, Presence presence) const
{
	QImage contactImage(*baseImage);
	QImage *pResultImage = new QImage(QSize::QSize(72,72), QImage::Format_ARGB32_Premultiplied);
		
	QImage presenceFrame;
    
	switch(presence)
	{
		case Hidden:
		case Offline:
			break;
			
		case Available:
			presenceFrame = QImage(":greenframe.png");
			break;
			
		case Busy:
			presenceFrame = QImage(":redframe.png");
			break;
			
		case XA:
		case Away:
			presenceFrame = QImage(":yellowframe.png");
	}
	
	QPainter painter(pResultImage);
	
	painter.setCompositionMode(QPainter::CompositionMode_Source);
    painter.fillRect(pResultImage->rect(), Qt::transparent);
    
    painter.setCompositionMode(QPainter::CompositionMode_SourceOver);
    painter.drawImage(0, 0, contactImage);

    painter.setCompositionMode(QPainter::CompositionMode_DestinationOver);
    painter.drawImage(0, 0, presenceFrame);
    painter.end();
    
    return pResultImage;
}
