/** \file TocAccountSettingsWidget.cpp
 * \brief Implementation of TocAccountSettingsWidget class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include <QMessageBox>
#include <QtDebug>

#include "TocAccountSettingsWidget"
#include "ui_TocAccountSettingsWidget.h"
#include "TocSettings"
#include "macros.h"



TocAccountSettingsWidget::TocAccountSettingsWidget(QWidget * parent, Qt::WindowFlags flags)
	: QWidget(parent, flags), _bCreateMode(false) {

	ui = new Ui_TocAccountSettingsWidgetClass();
	ui->setupUi( this );
	setContentsMargins( -10, -10, -10, -10 );
	setObjectName("accountSettings");

	ui->editVCardButton->hide();
	ui->changePasswordButton->hide();

	TocSettings* pSettings = TocSettings::getInstance();
	QStringList prot =  pSettings->protocols();
	for(int i = 0; i < prot.count(); ++i)
		ui->protocolComboBox->addItem(prot[i]);
	
	CONNECT(ui->backButton, SIGNAL(clicked()), this, SIGNAL(backClicked()));
}

TocAccountSettingsWidget::~TocAccountSettingsWidget() {
	delete ui;
}

void TocAccountSettingsWidget::loginEdited(const QString& newLogin) {
	TocSettings* pSettings = TocSettings::getInstance();
	ui->serverLineEdit->setText(pSettings->serverHint(newLogin, ui->protocolComboBox->currentText()));
	ui->portLineEdit->setText(QString("%1").arg(pSettings->portHint(newLogin, ui->protocolComboBox->currentText())));
}

void TocAccountSettingsWidget::protocolEdited(const QString& newProtocol) {
	TocSettings* pSettings = TocSettings::getInstance();
	ui->serverLineEdit->setText(pSettings->serverHint(ui->loginLineEdit->text(), newProtocol));
	ui->portLineEdit->setText(QString("%1").arg(pSettings->portHint(ui->loginLineEdit->text(), newProtocol)));
}

void TocAccountSettingsWidget::disconnectHints() {
	disconnect(ui->loginLineEdit, SIGNAL(textEdited(QString)), this, SLOT(loginEdited(QString)));
	disconnect(ui->protocolComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(protocolEdited(QString)));
	disconnect(ui->serverLineEdit, SIGNAL(textEdited(QString)), this, SLOT(disconnectHints()));
	disconnect(ui->portLineEdit, SIGNAL(textEdited(QString)), this, SLOT(disconnectHints()));
}

void TocAccountSettingsWidget::connectHints() {
	// disconnect first to ensure double connections are not made..
	disconnectHints();
	CONNECT(ui->loginLineEdit, SIGNAL(textEdited(QString)), this, SLOT(loginEdited(QString)));
	CONNECT(ui->protocolComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(protocolEdited(QString)));
	CONNECT(ui->serverLineEdit, SIGNAL(textEdited(QString)), this, SLOT(disconnectHints()));
	CONNECT(ui->portLineEdit, SIGNAL(textEdited(QString)), this, SLOT(disconnectHints()));

}

void TocAccountSettingsWidget::reload() {
	connectHints();

	if(createMode()) {
		ui->loginLineEdit->setText(	"" );
		ui->passwordLineEdit->setText( "" );
		ui->passwordConfirmLineEdit->setText( "" );
		ui->serverLineEdit->setText( "" );
		ui->protocolComboBox->setCurrentIndex( -1 );
		ui->portLineEdit->setText( "" );
	} else {
		TocSettings* pSettings = TocSettings::getInstance();
		ui->loginLineEdit->setText(	pSettings->accountUid() );
		ui->passwordLineEdit->setText( pSettings->accountPasswd() );
		ui->serverLineEdit->setText( pSettings->server() );
		ui->portLineEdit->setText(QString("%1").arg( pSettings->port() ));
		QString s = pSettings->protocolCommonName();
		ui->protocolComboBox->setCurrentIndex(
				ui->protocolComboBox->findText( s, Qt::MatchExactly ));
	}
}

void TocAccountSettingsWidget::showEvent(QShowEvent *event){
	reload();
	QWidget::showEvent(event);
}

void TocAccountSettingsWidget::handleError(QString errorText) {
	QMessageBox::information( this, "Invalid value", errorText);
}

bool TocAccountSettingsWidget::createMode() const {
	return _bCreateMode;
}

void TocAccountSettingsWidget::setCreateMode(bool enable) {

	_bCreateMode = enable;

	ui->passwordConfirmLabel->setVisible(enable);
	ui->passwordConfirmLineEdit->setVisible(enable);
	ui->changePasswordButton->setVisible(!enable);

	if(enable) {
		ui->okButton->setText(tr("Create"));
		ui->backButton->setText(tr("Back"));
	} else {
		ui->okButton->setText(tr("Ok"));
		ui->backButton->setText(tr("Back"));
	}
}

bool TocAccountSettingsWidget::dataVerified() {
	if(!ui->loginLineEdit->text().length()) {
		handleError(tr("Please provide login"));
		return false;
	}
	if(!ui->passwordLineEdit->text().length()) {
		handleError(tr("Please provide password"));
		return false;
	}
	if(createMode() && ui->passwordLineEdit->text() != ui->passwordConfirmLineEdit->text()) {
		handleError(tr("Passwords do not match"));
		return false;
	}
	if(!ui->serverLineEdit->text().length()) {
		handleError(tr("Please provide server name"));
		return false;
	}
	if(!ui->portLineEdit->text().toInt()) {
		handleError(tr("Please provide port number"));
		return false;
	}

	return true;
}

void TocAccountSettingsWidget::onChangePasswordButton(){
	// TODO: finish implementation
}

void TocAccountSettingsWidget::onOkButton(){
	if(!dataVerified())
		return;

	if(createMode())
		createNewAccount();
	else {
		saveAccountData();
		emit finished(); // Close the view when not in `Create mode`
	}
}

void TocAccountSettingsWidget::saveAccountData() {

	bool settingsChanged = false;

	TocSettings* pSettings = TocSettings::getInstance();

	if(ui->loginLineEdit->text() != pSettings->accountUid()) {
		pSettings->setAccountUid(ui->loginLineEdit->text());
		settingsChanged = true;
	}

	if(ui->passwordLineEdit->text() != pSettings->accountPasswd()) {
		pSettings->setAccountPasswd(ui->passwordLineEdit->text());
//		settingsChanged = true; // This is not needed for only the password
	}

	if(ui->protocolComboBox->currentText() != pSettings->protocolCommonName()) {
		pSettings->setProtocolByName(ui->protocolComboBox->currentText());
		settingsChanged = true;
	}

	if(ui->serverLineEdit->text() != pSettings->server()) {
		pSettings->setServer(ui->serverLineEdit->text());
		settingsChanged = true;
	}

	if(ui->portLineEdit->text().toInt() != pSettings->port()) {
		pSettings->setPort(ui->portLineEdit->text().toInt());
		settingsChanged = true;
	}

	if(settingsChanged) {
		emit accountSettingsChanged();
	}
}

void TocAccountSettingsWidget::createNewAccount() {

	TocSettings* pSettings = TocSettings::getInstance();
	pSettings->setAccountUid(ui->loginLineEdit->text());
	pSettings->setAccountPasswd(ui->passwordLineEdit->text());
	pSettings->setServer(ui->serverLineEdit->text());
	pSettings->setPort(ui->portLineEdit->text().toInt());
	pSettings->setProtocolByName(ui->protocolComboBox->currentText());
	pSettings->setRegister();

	emit newAccountSettingsCreated();
}
