/** \file TocChatWidget.cpp
 * \brief Implementation of TocChatWidget class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include <QDateTime>
#include <QKeyEvent>

#include "ui_TocChatWidget.h"
#include "TocEmoticonsDialog"
#include "TocChatWidget"
#include "uidefs.h"
#include "macros.h"

TocChatWidget::TocChatWidget( const QString& uid, const QString& name, Presence presence, QWidget * parent, const Message& message )
	: QWidget(parent) 
{
	ui = new Ui_TocChatWidgetClass();
	ui->setupUi( this );
	setContentsMargins( -10, -10, -10, -10 );
	
	CONNECT(ui->smileButton,SIGNAL(clicked()),this,SLOT(showEmoticonsDialog()));
	
	_uid = uid;
	_name = name;
	
	switch(presence)
	{
	case Available:
		ui->frame->setStyleSheet(STYLESHEET_ONLINE);
		break;
	case XA:
	case Away:
		ui->frame->setStyleSheet(STYLESHEET_AWAY);
		break;
	case Busy:
		ui->frame->setStyleSheet(STYLESHEET_BUSY);
		break;
	case Hidden:	
	case Offline:  
		ui->frame->setStyleSheet(STYLESHEET_OFFLINE);
		break;
	default:
		ui->frame->setStyleSheet(STYLESHEET_OFFLINE);
	}

	appendMessage(message);
	
	//Event filter for sending on Enter support  
	ui->messageTextEdit->installEventFilter(this);
	ui->messageTextEdit->setFocus();
}

TocChatWidget::~TocChatWidget() {
	delete ui;
}

void TocChatWidget::onContactPresenceUpdate(Presence presence, const QString& description) {
	QFont font = ui->chatTextEdit->currentFont();
	QFont smallFont;
	smallFont.setPointSize(static_cast<int>( font.pointSize() * 0.85) );
	smallFont.setItalic(true);
	ui->chatTextEdit->setCurrentFont(smallFont);
	ui->chatTextEdit->setTextColor( Qt::darkGreen );
	
	QString status;
	switch(presence)
	{
		case Available:
			status = tr("Online"); 	
			ui->frame->setStyleSheet(STYLESHEET_ONLINE);
			break;
		case XA:
		case Away:
			status = tr("Away"); 
			ui->frame->setStyleSheet(STYLESHEET_AWAY);
			break;
		case Busy:
			status = tr("Do not Disturb"); 
			ui->frame->setStyleSheet(STYLESHEET_BUSY);
			break;
		case Hidden:
		case Offline:
			status = tr("Offline"); 
			ui->frame->setStyleSheet(STYLESHEET_OFFLINE);
			break;
		default:
			status = tr("Unknown"); 
			ui->frame->setStyleSheet(STYLESHEET_OFFLINE);
	}
	
	if(description.size())
		ui->chatTextEdit->append(tr("%1 changed status to %2 (%3)").arg(_name).arg(status).arg(description));
	else
		ui->chatTextEdit->append(tr("%1 changed status to %2").arg(_name).arg(status));
	
	ui->chatTextEdit->moveCursor(QTextCursor::End);
	ui->chatTextEdit->setCurrentFont(font);
}

void TocChatWidget::updateContactPresence(Presence presence) {
	switch(presence)
	{
	case Available:
		ui->frame->setStyleSheet(STYLESHEET_ONLINE);
		break;
	case XA:
	case Away:
		ui->frame->setStyleSheet(STYLESHEET_AWAY);
		break;
	case Busy:
		ui->frame->setStyleSheet(STYLESHEET_BUSY);
		break;
	case Hidden:	
	case Offline:  
		ui->frame->setStyleSheet(STYLESHEET_OFFLINE);
		break;
	default:
		ui->frame->setStyleSheet(STYLESHEET_OFFLINE);
	}
}

void TocChatWidget::appendMessage(const Message& message) {
	if(message.contents.isEmpty())
		return;

	QDateTime messageTime( QDateTime::fromTime_t( message.timestamp ) );
	QDateTime currentTime( QDateTime::currentDateTime() );

	QString format;
	if( messageTime.date() < currentTime.date() )
		format = "ddd: yyyy-MM-dd hh:mm:ss";
	else
		format = "hh:mm:ss";
	
	if(message.error == NoError) {

		ui->chatTextEdit->setTextColor(Qt::blue);
		ui->chatTextEdit->setFontItalic(true);
		ui->chatTextEdit->append( _name + " " + currentTime.toString("hh:mm:ss")
				+ " (" + messageTime.toString(format) + "): ");
		ui->chatTextEdit->setFontItalic(false);
		insertEmoticons(message.contents);
		
	} else { 
		
		// If error occured
		ui->chatTextEdit->setTextColor(Qt::red);
		ui->chatTextEdit->setFontItalic(true);
		ui->chatTextEdit->append( tr("Message \"%1\" sent on %2 was not delivered %3"
				).arg(message.contents).arg(messageTime.toString(format)).arg(errorString(message.error)));
	}
	
	ui->chatTextEdit->moveCursor(QTextCursor::End);
}

QString TocChatWidget::errorString(MessageDeliveryError error) {
	
	switch(error) {
	case Unknown:
		return tr("because of unknown error.");

	case ContactOffline:
		return tr("because contact was offline.");

	case InvalidContact:
		return tr("because contact is invalid.");

	case PermissionDenied:
		return tr("because permission was denied.");

	case MessageTooLong:
		return tr("because message was too long.");

	case UserOffline:
		return tr("because You are offline, connect to be able to send messages.");

	case NotOnContactList:
		return tr("because the contact is not on Your contact list.");
	
	case CannotCreateChannel:
		return tr("because the channel with the contact could not be created.");

	case NoError:
		return tr("because there was no error :).");

	default:
		return tr("because of unknown error.");
	}
}

void TocChatWidget::pressedSendButton() {
	
	if(ui->messageTextEdit->toPlainText().size()) {
		QDateTime time(QDateTime::currentDateTime());

		ui->chatTextEdit->setTextColor(Qt::black);
		ui->chatTextEdit->setFontItalic(true);
		ui->chatTextEdit->append( tr("me") + " (" + time.toString("hh:mm:ss") + "): " );
		ui->chatTextEdit->setFontItalic(false);
		
		insertEmoticons(ui->messageTextEdit->toPlainText());
		
		Message message;
		message.contents = ui->messageTextEdit->toPlainText();
		message.timestamp = time.toTime_t();
		emit newMessageReady(_uid, message);
		ui->messageTextEdit->clear();
		
		ui->chatTextEdit->moveCursor(QTextCursor::End);
	}
} 

bool TocChatWidget::eventFilter(QObject *object, QEvent *event) {
	
	if (object == ui->messageTextEdit) {
		
		if (event->type() == QEvent::KeyPress) {
			
			QKeyEvent *k = static_cast<QKeyEvent*>(event);				
			if (k->key() == Qt::Key_Enter || k->key() == Qt::Key_Return) {
				
				if ((!(k->modifiers() == (Qt::KeypadModifier | Qt::ShiftModifier)))	&& 
					(!(k->modifiers() == Qt::ShiftModifier))) {
					
					pressedSendButton();			
					return true;
				}
			}
		}
	}
	return false;
}

void TocChatWidget::insertEmoticons(const QString& message) {
	
    int pos = 0, last = 0, delta = 1;
    QString emot;
   
    while (pos+1 < message.length()) {
    	
        // checking for ":)" or ":-)" type smilies...
        if(message.at(pos) == ':') {

            if(message.at(pos+1) == '-' && pos+2 < message.length()) { // checking for :-) type...
                delta = 2;
            }
            if(message.at(pos+delta) == ')') {
                delta++; emot = emotSmile_24x24;
            } 
            else if(message.at(pos+delta) == '(') {
                delta++; emot = emotSad_24x24;
            }
            else if(message.at(pos+delta) == '|') {
                delta++; emot = emotPlain_24x24;
            }
            else if(message.at(pos+delta) == 'D') {
                delta++; emot = emotGrin_24x24;
            }
            else if(message.at(pos+delta) == 'O') {
                delta++; emot = emotSurprise_24x24;
            }
            else if(message.at(pos+delta) == '*') {
                delta++; emot = emotKiss_24x24;
            }
            else
                delta = 1;
        }
        
        // checking for ";)" or ";-)" type smilies...
        else if(message.at(pos) == ';') {
        	
            if(message.at(pos+1) == '-' && pos+2 < message.length()) { // checking for ;-) type...
                delta = 2;
            }
            if(message.at(pos+delta) == ')') {
                delta++; emot = emotWink_24x24;
            }
            else
                delta = 1;
        }
        
        // checking for "<emoticon>" type smilies...
        else if(message.at(pos) == '<') {
        	
            if(message.right(message.length()-pos).startsWith("<angel>")) {
                delta = 7; emot = emotAngel_24x24;
            }
            else if(message.right(message.length()-pos).startsWith("<devil>")) {
                delta = 7; emot = emotDevil_24x24;
            }
            else if(message.right(message.length()-pos).startsWith("<monkey>")) {
                delta = 8; emot = emotMonkey_24x24;
            } 
            else if(message.right(message.length()-pos).startsWith("<crying>")) {
                delta = 8; emot = emotCrying_24x24;
            }   
            else if(message.right(message.length()-pos).startsWith("<glasses>")) {
                delta = 9; emot = emotGlasses_24x24;
            }   
            else if(message.right(message.length()-pos).startsWith("<bigsmile>")) {
                delta = 10; emot = emotSmileBig_24x24;
            }   
            else
                delta = 1;
        }
        
        //OK, emoticon found ... inserting into chatView
        if(delta>1) {
            ui->chatTextEdit->insertPlainText(message.mid(last, pos-last));
            ui->chatTextEdit->insertHtml("<img source=\"" + emot + "\">");
            pos += delta; last = pos; delta = 1;
        }
        else {
            pos += delta;
        }
    }
    ui->chatTextEdit->insertPlainText(message.mid(last));
}

void TocChatWidget::showEmoticonsDialog() {
	
	TocEmoticonsDialog* emoticonsDialog = TocEmoticonsDialog::getInstance(this);

	if ((*emoticonsDialog).exec() == QDialog::Accepted) {
	
		QString alias = emoticonsDialog->getSelectedEmoticonAlias();	
		ui->messageTextEdit->insertPlainText(alias);
	}
	ui->messageTextEdit->setFocus();
}

void TocChatWidget::showEvent( QShowEvent* event ) {
	ui->messageTextEdit->setFocus();
	event->accept();
}
