/** \file TocContactListWidget.cpp
 * \brief Implementation of TocContactListWidget class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <QMenu>
#include <QMouseEvent>
#include <QMessageBox>
#include <QPainter>
#include <QDebug>
#include <QCommonStyle>

#include "TocContactListWidget"
#include "IconProvider"
#include "uidefs.h"
#include "macros.h"

Q_DECLARE_METATYPE(TocContact*);

TocContactListWidget::TocContactListWidget(QWidget * parent)
	: QListWidget(parent),
	_editAction(tr("Edit"), 0),
	_removeAction(tr("Remove"), 0)
{
	setStyle(new QCommonStyle());
	setFlow( QListView::LeftToRight );
	setViewMode( QListView::IconMode );
	setGridSize( QSize(109, 113) );
	setIconSize(QSize(68,68));
	setDragEnabled( false );
	setVerticalScrollMode( QAbstractItemView::ScrollPerItem );
	setWrapping( true );
	setWordWrap( true );
	setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
	setResizeMode( QListView::Adjust );
	QFont currentFont(font());
	currentFont.setPixelSize(14);
	setFont(currentFont);

	// 'Add contact' item
	(new QListWidgetItem(QIcon(":/addcontact.png"), tr("Add Contact"), this))->setFlags( Qt::ItemIsEnabled );

	TocSettings *pSettings = TocSettings::getInstance();
	
	QList<TocContact*> *pContactsList = pSettings->contactList();
	
	IconProvider *pIconProvider = IconProvider::getInstance();
	
	foreach(TocContact* pContact, *pContactsList)
	{
		if(!pContact->customIconPath.isEmpty())
			pIconProvider->setCustomIcon(pContact);

		QString displayedText;
		if(pContact->name.isEmpty())
			displayedText = pContact->uid;
		else
			displayedText = pContact->name;

		QListWidgetItem* pItem = new QListWidgetItem(displayedText, this);
		
		pItem->setData( Qt::DecorationRole, *(pIconProvider->matchIcon(pContact)) );
		pItem->setData( ContactRole, QVariant::fromValue(pContact));
		
		pItem->setFlags( Qt::ItemIsEnabled );
			
		addItem(pItem);
	}
	
	// Load logo image
	_logo = new QImage(":/tieto-logo.png");

	sortItems(Alphabetical); // Sort items alphabetically

	//CONNECT SIGNALS
	CONNECT(&_editAction, SIGNAL(triggered(bool)), this, SLOT(onEditRequested()));
	CONNECT(&_removeAction, SIGNAL(triggered(bool)), this, SLOT(onRemoveRequested()));
	CONNECT(this, SIGNAL(logoColorChanged(QColor)), this, SLOT(changeLogoColor(QColor)));
}

TocContactListWidget::~TocContactListWidget() {
	delete _logo;
}

void TocContactListWidget::onNewContactList(const TocContactList& contactList) {

	QListWidgetItem* pAddContactItem = takeItem( 0 );
	clear();

	IconProvider *pIconProvider = IconProvider::getInstance();	
	
	foreach(TocContact* pContact, contactList)
	{
		QString displayedText;
		if(pContact->name.isEmpty())
			displayedText = pContact->uid;
		else
			displayedText = pContact->name;

		QListWidgetItem* pNewItem = new QListWidgetItem(displayedText, this);

		pNewItem->setData(DecorationRole, *(pIconProvider->matchIcon(pContact)));
		pNewItem->setData(ContactRole, QVariant::fromValue(pContact));

		//make it not selectable
		pNewItem->setFlags( Qt::ItemIsEnabled );
	}

	insertItem(0, pAddContactItem);

	sortItems(Alphabetical);
}

void TocContactListWidget::onContactPresenceUpdate(const QString& uid, Presence presence, const QString& description) {

	IconProvider *pIconProvider = IconProvider::getInstance();

	QListWidgetItem* pItem = findItem(uid);

	if( !pItem )
		return;
	
	TocContact *pContact = pItem->data(ContactRole).value<TocContact*>();

	pContact->presence = presence;
	pContact->description = description;

	pItem->setData(Qt::DecorationRole, *(pIconProvider->matchIcon(pContact)));
}

QString TocContactListWidget::getDisplayedName(const QString& uid) {
	
	QListWidgetItem* pItem = findItem(uid);
	
	if(pItem)
		return pItem->data(DisplayRole).toString();
	
	return QString();
}

void TocContactListWidget::clearStatuses() {

	IconProvider *pIconProvider = IconProvider::getInstance();

	for(int i = 1; i < count(); ++i) {

		QListWidgetItem* pItem = this->item(i);

		TocContact *pContact = pItem->data(ContactRole).value<TocContact*>();
		pContact->presence = Offline;
//		pContact->description = ""; // TODO: Clear description also? Some clients remember last descr

		pItem->setData(Qt::DecorationRole, *(pIconProvider->matchIcon(pContact)));
	}
}

Q_DECLARE_METATYPE(QListWidgetItem*);


void TocContactListWidget::mouseDoubleClickEvent(QMouseEvent *event)
{
	QPoint itemPos = event->pos();
	
	QListWidgetItem* pClickedItem = itemAt(itemPos);
	if(!pClickedItem)
		return;

	QListWidgetItem* pAddContactItem = item(0);

	if(pAddContactItem == pClickedItem)
		emit contactInfoRequest(0);
	else
	{
		TocContact *pContact = pClickedItem->data(ContactRole).value<TocContact*>();
		QString uid = pContact->uid;
		QString name = pClickedItem->text(); // Using Text instead of pContact->name because the latter can be empty

		emit triggered(uid, name, pContact->presence);
	}
}

void TocContactListWidget::contextMenuEvent(QContextMenuEvent *event)
{
	QPoint itemPos = event->pos();
	
	QListWidgetItem* pClickedItem = itemAt(itemPos);
		
	//if it applies to Add Contact then skip it
	QListWidgetItem* pAddContactItem = item(0);
	if(pClickedItem == pAddContactItem || pClickedItem == 0) return;
				
	//create menu
	QMenu contextMenu(this);

	contextMenu.setStyleSheet(
			"QMenu {"
				"margin-left: 0px;"
				"margin-top: 0px;"
				"margin-right: 0px;"
				"margin-bottom: 0px;"
				"font: bold 16px;"
			"}"
			"QMenu::item {"
				"margin-left: 0px;"
				"margin-top: 0px;"
				"margin-right: 0px;"
				"margin-bottom: 0px;"
				"padding: 0px 10px 0px 30px;"
				"min-height: 60px;"
				"min-width: 80px;"
				"max-height: 60px;"
				"border-radius: 5px;"
			"}");
	
	_editAction.setData(QVariant::fromValue(pClickedItem));
	_removeAction.setData(QVariant::fromValue(pClickedItem));
	
	contextMenu.addAction(&_editAction);
	contextMenu.addAction(&_removeAction);
	
	contextMenu.exec(mapToGlobal(itemPos));
}


void TocContactListWidget::onEditRequested() {

	//extract data from sender
	QObject* pObject = QObject::sender();
	_pCurrentEditedItem = (static_cast<QAction*>(pObject))->data().value<QListWidgetItem*>();

	_currentEditedContact = *(_pCurrentEditedItem->data(ContactRole).value<TocContact*>());

	if(_pCurrentEditedItem)
		emit contactInfoRequest(_pCurrentEditedItem);
}

void TocContactListWidget::onRemoveRequested() {
	
	//extract data from sender
	QObject* pObject = QObject::sender();
	QListWidgetItem* pItem = (static_cast<QAction*>(pObject))->data().value<QListWidgetItem*>();
	
	if(!pItem) return;
	
	TocContact* pContact = pItem->data(ContactRole).value<TocContact*>();
	QMessageBox::StandardButton clickedButton = QMessageBox::question(this, "TEC", tr("Are you sure you want to remove\ncontact: %1").arg(pContact->name), QMessageBox::Yes | QMessageBox::No);
	if(clickedButton == QMessageBox::No) return;
	
	//Remove from list
	if(takeItem(row(pItem)))
	{
		//Removing went successfuly
		//Lets notify the engine
		QString uid = pItem->data(ContactRole).value<TocContact*>()->uid;
		
		emit removedContact(uid);
	}
	
}

void TocContactListWidget::commitAdd(QListWidgetItem* pNewItem) {
	
	TocContact *pNewContact = pNewItem->data(ContactRole).value<TocContact*>();
	QString uid = pNewContact->uid;
	
	//ensure that item is not already present
	if(findItem(uid))
		return;
	
	//add to list
	addItem(pNewItem);

	// sortItems now
	sortItems(Alphabetical);
	
	//notify engine
	emit addContact(pNewContact);
}

void TocContactListWidget::commitEdit(QListWidgetItem* pEditedItem) {

	TocContact* pEditedContact = pEditedItem->data(ContactRole).value<TocContact*>();

	QString newUid = pEditedContact->uid;
	QString newName = pEditedContact->name;

	QString oldUid = _currentEditedContact.uid;
	QString oldName = _currentEditedContact.name;
	
	if( newUid != oldUid )//If Uid has changed ensure that item with such uid is not already present
		if( findItem(newUid, pEditedItem) ) {
			pEditedContact->uid = oldUid;
			return;
		}

	if( newName != oldName )
		if(newName.isEmpty())
			emit displayedNameChanged( newUid, newUid );
		else
			emit displayedNameChanged( newUid, newName );

	// sort items now
	sortItems(Alphabetical);

	IconProvider *pIconProvider = IconProvider::getInstance();
	pEditedItem->setData(Qt::DecorationRole, *(pIconProvider->matchIcon(pEditedContact)));

	emit editContact(pEditedContact, oldUid);
}

QListWidgetItem* TocContactListWidget::findItem(const QString& requestedUid, const QListWidgetItem* pExcludedItem) {
	
	QString uid;
	
	for(int i=1; i < count(); i++)
	{
		QListWidgetItem* pRequestedItem = item(i);
		if( pExcludedItem == pRequestedItem )
			continue;

		uid = pRequestedItem->data(ContactRole).value<TocContact*>()->uid;
		
		if(uid == requestedUid)
			return pRequestedItem;
	}
		
	return 0;
}

void TocContactListWidget::sortItems(SortOrder order) {
	
	QListWidgetItem* pAddContactItem = 0;
	
	switch(order) {
	
		case Alphabetical:
			pAddContactItem = takeItem( 0 );
			QListWidget::sortItems(Qt::AscendingOrder);
			insertItem(0, pAddContactItem);
			break;

		case ByStatus:
			// TODO: sort by status
			break;

		case None:
			// TODO: disable sorting and enable drag & drop events
			break;
	}
}

void TocContactListWidget::paintEvent(QPaintEvent * event) {
	Q_UNUSED(event);

	QPainter painter(this->viewport());
	painter.drawImage(this->width() - _logo->width(), this->height() - _logo->height(), *_logo);

	QListWidget::paintEvent(event);
}

QColor TocContactListWidget::logoColor() const {
	return _logoColor;
}

void TocContactListWidget::setLogoColor(QColor color) {
	_logoColor = color;
	emit logoColorChanged( _logoColor );
}

void TocContactListWidget::changeLogoColor(QColor color) {
	QColor lColor;
	QString colorName = color.name();

	if( colorName == "#0000ff" )
	      lColor = LOGO_BLUE;
	else if( colorName == "#add8e6" )
	      lColor = LOGO_LIGHT_BLUE;
	else if( colorName == "#008000" )
	      lColor = LOGO_GREEN;
	else if( colorName == "#ffa500" )
	      lColor = LOGO_ORANGE;
	else if( colorName == "#ff0000" )
	      lColor = LOGO_RED;
	else
	      lColor = LOGO_RED;

	for(int i = 0; i < _logo->width(); ++i)
		for(int j = 0; j < _logo->height(); ++j)
			 if(_logo->pixel(i, j) >> 24)
				_logo->setPixel(i,j, lColor.rgb());
}
