/** \file TocMainToolbar.cpp
 * \brief Implementation of TocMainToolbar class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
 
#include "TocMainToolbar"
#include "macros.h"
#include <QPushButton> 
#include <QHBoxLayout>
#include <QMenu>
#include <QLabel>
#include <QEvent>
#include <QDebug>
#include <QThread>


TocMainToolbar::TocMainToolbar(QWidget * parent, Qt::WindowFlags flags)
	: QWidget(parent, flags) {
	
	setObjectName("mainToolbar");
	QHBoxLayout* horizontalLayout = new QHBoxLayout( this );
	horizontalLayout->setSpacing( 0 );
	setContentsMargins( -10, -10, -10, -10 );

	statusButton = new QPushButton( QIcon(":/offline.png"), "", this );
	statusButton->setObjectName("statusButton");
	statusButton->setIconSize( QSize( 72, 72 ) );
	horizontalLayout->addWidget( statusButton );
	statusMenu = new QMenu( this );
	statusMenu->setObjectName("statusMenu");
	onlineAction = statusMenu->addAction( QIcon(":/online.png"), tr("Online") );
	onlineAction->setData( QVariant::fromValue( Available ) );
	awayAction = statusMenu->addAction( QIcon(":/away.png"), tr("Away") );
	awayAction->setData( QVariant::fromValue( Away ) );
	busyAction = statusMenu->addAction( QIcon(":/busy.png"), tr("Do not Disturb") );
	busyAction->setData( QVariant::fromValue( Busy ) );
	hiddenAction = statusMenu->addAction( QIcon(":/hidden.png"), tr("Hidden") );
	hiddenAction->setData( QVariant::fromValue( Hidden ) );
	offlineAction = statusMenu->addAction( QIcon(":/offline.png"), tr("Offline") );
	offlineAction->setData( QVariant::fromValue( Offline ) );
	statusButton->setMenu( statusMenu );
	CONNECT( statusMenu, SIGNAL(triggered(QAction*)), this, SLOT(onTriggeredPresence(QAction*)) );

	statusAreaLabel = new QLabel( this );
	statusAreaLabel->setObjectName("statusAreaLabel");
	statusAreaLabel->setWordWrap( true );
	statusAreaLabel->setProperty("checked", false);
	statusAreaLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
	horizontalLayout->addWidget( statusAreaLabel );

	statusAreaButton = new QPushButton( this );
	statusAreaButton->setObjectName("statusAreaButton");
	statusAreaButton->setEnabled( false );
	statusAreaButton->setCheckable( true );
	horizontalLayout->addWidget( statusAreaButton );

	activeSessionsButton = new QPushButton( QIcon(":/activesessions.png"), "", this );
	activeSessionsButton->setObjectName("activeSessionsButton");
	activeSessionsButton->setIconSize( QSize( 72, 72 ) );
	activeSessionsButton->setEnabled(false);
	horizontalLayout->addWidget( activeSessionsButton );
	

	_pActiveSessions = new ActiveSessions(this);
	
	activeSessionsButton->setMenu( _pActiveSessions->menu() );
	activeSessionsButton->setObjectName("activeSessionsButton");
	
	settingsButton = new QPushButton( QIcon(":/settings.png"), "", this );
	settingsButton->setObjectName("settingsButton");
	settingsButton->setIconSize( QSize( 72, 72 ) );
	settingsButton->setCheckable( true );
	horizontalLayout->addWidget( settingsButton );
	CONNECT( settingsButton, SIGNAL(clicked()), this, SLOT(onSettingsClicked()) );

	contactsButton = new QPushButton( QIcon(":/contacts.png"), "", this );
	contactsButton->setObjectName("contactsButton");
	contactsButton->setIconSize( QSize( 72, 72 ) );
	contactsButton->setCheckable( true );

	horizontalLayout->addWidget( contactsButton );
	CONNECT( contactsButton, SIGNAL(clicked()), this, SLOT(onContactsClicked()) );
	highlightButton(contactsButton);

	setupStyles();
}

TocMainToolbar::~TocMainToolbar() {
}

void TocMainToolbar::highlightButton(const QPushButton* button) {
	
	contactsButton->setChecked(button == contactsButton);
	settingsButton->setChecked(button == settingsButton);
	statusAreaButton->setChecked(button == statusAreaButton);
	statusAreaLabel->setEnabled(button == statusAreaButton);
}

void TocMainToolbar::restoreCurrentSession() {
	
	if( QAction* action = _pActiveSessions->disabledAction() ) {
		
		highlightButton(statusAreaButton);
		
		emit activeSessionTriggered( action->data().toString(), action->text() );
		
		statusAreaButton->setIcon( QIcon(":/close.png"));
		
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
		CONNECT( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
	}
}

void TocMainToolbar::closeCurrentSession() {
	qDebug() << __PRETTY_FUNCTION__;
	
	QAction* pAction = _pActiveSessions->disabledAction();
	
	if( pAction ) {
	
		QAction *pNewCurrentAction = _pActiveSessions->closeCurrentSession();
	
		if( pNewCurrentAction ) {
		
			statusAreaButton->setChecked(true);
			statusAreaLabel->setText( pNewCurrentAction->text() );
		
			emit activeSessionTriggered( pNewCurrentAction->data().toString(), pNewCurrentAction->text() );
	
		} else {
		
			activeSessionsButton->setEnabled(false);
			statusAreaLabel->setText("");
			statusAreaButton->setIcon( QIcon() );
			statusAreaButton->setEnabled( false );
			disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
			highlightButton(contactsButton);
			emit contactsClicked();
		}
		
		emit finishedSession( pAction->data().toString() );
	}
		
}

void TocMainToolbar::closeAllSessions() {
	
	activeSessionsButton->setEnabled(false);
		
	_pActiveSessions->clear();
	
	statusAreaLabel->setText("");
	
	statusAreaButton->setIcon( QIcon() );
	statusAreaButton->setEnabled( false );
	
	disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
	disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
	
	emit finishedAllSessions();
	
	highlightButton(contactsButton);
	
	emit contactsClicked();
}

void TocMainToolbar::clearAllSessions() {

	activeSessionsButton->setEnabled(false);
	_pActiveSessions->clear();

	statusAreaLabel->setText("");
	statusAreaButton->setIcon( QIcon() );
	statusAreaButton->setEnabled( false );
	
	disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
	disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );	
}


void TocMainToolbar::onTriggeredPresence(QAction* action) {
	
	emit presenceUpdate( action->data().value<Presence>(), "opisik" );
}

void TocMainToolbar::onPresenceUpdate(Presence presence, const QString& description) {
	
	Q_UNUSED(description);
	
	if( presence == onlineAction->data().value<Presence>() )
		statusButton->setIcon( onlineAction->icon() );
	
	else if( presence == awayAction->data().value<Presence>() )
		statusButton->setIcon( awayAction->icon() );
	
	else if( presence == busyAction->data().value<Presence>() )
		statusButton->setIcon( busyAction->icon() );
	
	else if( presence == hiddenAction->data().value<Presence>() )
		statusButton->setIcon( hiddenAction->icon() );
	
	else if( presence == offlineAction->data().value<Presence>() )
		statusButton->setIcon( offlineAction->icon() );
}

void TocMainToolbar::onContactPresenceUpdate(const QString& uid, Presence presence) {
	
	_pActiveSessions->onContactPresenceUpdate(uid, presence);

}

void TocMainToolbar::onNewMessage(const QString& uid, const QString& name, Presence presence) {
	
	qDebug()<<__PRETTY_FUNCTION__;
	
	if( QAction* pAction = _pActiveSessions->action( uid ) ) {
		
		if( pAction != _pActiveSessions->disabledAction() ) {

			activeSessionsButton->setChecked( true );
			//TODO: activeSessionsButton->startBlinking()
			
		} else {
			
			if( statusAreaButton->icon().cacheKey() == QIcon( ":/restore.png" ).cacheKey() )//
				statusAreaButton->setChecked( true );
		}
		
	} else {
		
		activeSessionsButton->setEnabled(true);
		//TODO: activeSessionsButton->startBlinking()
	}
	
	_pActiveSessions->onNewMessage( uid, name, presence );
}

void TocMainToolbar::onNewSession(const QString& uid, const QString& name, Presence presence) {

	qDebug()<<__PRETTY_FUNCTION__;
	
	highlightButton(statusAreaButton);

	QAction* pDisabledAction = _pActiveSessions->disabledAction(); 
	
	if( QAction* pAction = _pActiveSessions->action( uid ) ) {
		if( pAction != pDisabledAction ) {

			statusAreaLabel->setText( pAction->text() );
			statusAreaButton->setIcon( QIcon(":/close.png"));
			statusAreaButton->setEnabled( true );
			
			disconnect( statusAreaButton, SIGNAL(clicked()), 
									this, SLOT(restoreCurrentSession()) );
			disconnect( statusAreaButton, SIGNAL(clicked()),
									this, SLOT(closeCurrentSession()) );
			CONNECT( statusAreaButton, SIGNAL(clicked()),
									this, SLOT(closeCurrentSession()) );

		} else {

			statusAreaButton->setIcon( QIcon(":/close.png"));
			
			disconnect( statusAreaButton, SIGNAL(clicked()),
									this, SLOT(restoreCurrentSession()) );
			disconnect( statusAreaButton, SIGNAL(clicked()),
									this, SLOT(closeCurrentSession()) );
			CONNECT( statusAreaButton, SIGNAL(clicked()),
									this, SLOT(closeCurrentSession()) );
		}
		
	} else {

		statusAreaLabel->setText( name );
		
		statusAreaButton->setIcon( QIcon(":/close.png"));
		statusAreaButton->setEnabled( true );
		activeSessionsButton->setEnabled(true);
		
		disconnect( statusAreaButton, SIGNAL(clicked()),
								this, SLOT(restoreCurrentSession()) );
		disconnect( statusAreaButton, SIGNAL(clicked()),
								this, SLOT(closeCurrentSession()) );
		CONNECT( statusAreaButton, SIGNAL(clicked()),
								this, SLOT(closeCurrentSession()) );
	}
	
	_pActiveSessions->onNewSession( uid, name, presence );
	
}

void TocMainToolbar::onCurrentView(ViewType viewType, const QString& text) {
	
	if( QAction* pAction = _pActiveSessions->disabledAction() ) { // If there is an active window
		
		if( viewType == Contacts ) {
			
			statusAreaButton->setIcon( QIcon(":/restore.png") );
			disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
			disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
			CONNECT( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
			highlightButton(contactsButton);
			
		} else {
			pAction->setEnabled( true );
			statusAreaLabel->setText( text );
			statusAreaButton->setIcon( QIcon() );
			statusAreaButton->setEnabled( false );
			disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
			disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
			// Deselect highlight from all buttons but leave settingsButton as is (highlighted or not)
			contactsButton->setChecked(false);
			contactsButton->setChecked(false);
			statusAreaButton->setChecked(false);
			statusAreaLabel->setEnabled(false);
		}
	} else if( viewType == Contacts ) {
		highlightButton(contactsButton);
		statusAreaLabel->setText( text );	
	}
	else { // If there is no conversation at that time and the viewType != contacts
		statusAreaLabel->setText( text );
		statusAreaButton->setIcon( QIcon() );
		statusAreaButton->setEnabled( false );
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
		// Deselect highlight from all buttons but leave settingsButton as is (highlighted or not)
		contactsButton->setChecked(false);
		contactsButton->setChecked(false);
		statusAreaButton->setChecked(false);
		statusAreaLabel->setEnabled(false);
	}
}

void TocMainToolbar::onDisplayedNameChange(const QString& uid, const QString& name) {
	
	if( QAction* pAction = _pActiveSessions->action( uid ) )
		pAction->setText( name );
}

void TocMainToolbar::onContactsClicked() {
	
	highlightButton(contactsButton);

	if( _pActiveSessions->disabledAction() ) {
		
		statusAreaButton->setIcon( QIcon(":/restore.png") );
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
		CONNECT( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
		
	} else {
		
		statusAreaLabel->setText("");
		statusAreaButton->setIcon( QIcon() );
		statusAreaButton->setEnabled( false );
	}

	emit contactsClicked();
}

void TocMainToolbar::onSettingsClicked() {
	
	highlightButton(settingsButton);

	if( QAction* pAction = _pActiveSessions->disabledAction() ) {
		
		pAction->setEnabled( true );
		
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
		disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );		
	}

	statusAreaLabel->setText( tr("Settings") );
	statusAreaButton->setIcon( QIcon() );
	statusAreaButton->setEnabled( false );

	emit settingsClicked();
}

void TocMainToolbar::onActiveSessionTriggered(QAction* pAction)
{
	if(pAction->text() == "")
		return;

	if( QAction* pDisabledAction = _pActiveSessions->disabledAction() )
		pDisabledAction->setEnabled( true );
	
	pAction->setChecked( false );
	pAction->setEnabled( false );
	
	statusAreaLabel->setText( pAction->text() );
	statusAreaButton->setIcon( QIcon(":/close.png"));
	statusAreaButton->setEnabled( true );
	
	disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(restoreCurrentSession()) );
	disconnect( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
	CONNECT( statusAreaButton, SIGNAL(clicked()), this, SLOT(closeCurrentSession()) );
	
	highlightButton(statusAreaButton);
	
	emit activeSessionTriggered( pAction->data().toString(), pAction->text() );	
}


void TocMainToolbar::setupStyles() {

	statusButton->setStyleSheet(
			"QPushButton {\
    border: none;\
	min-height: 80px;\
	min-width: 100px;\
	max-height: 80px;\
	max-width: 100px;\
 }"
	);

	statusAreaLabel->setStyleSheet(" QLabel {\
	font: bold 20px; \
	border: none;\
	border-radius: 0px;\
	min-height: 80px;\
	min-width: 100px;\
	max-height: 80px;\
 }");
	
	statusAreaButton->setStyleSheet(" QPushButton {\
    border: none;\
	border-radius: 0px;\
  	min-height: 80px;\
	min-width: 80px;\
	max-height: 80px;\
	max-width: 80px;\
 }");

	activeSessionsButton->setStyleSheet(" QPushButton {\
    border: none;\
  	min-height: 80px;\
	min-width: 100px;\
	max-height: 80px;\
	max-width: 100px;"
"}"
	" QPushButton:checked {\
			background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,\
			                        stop: 0 #999900, stop: 0.55 #aaaa00,\
						            stop: 0.56 #bbbb00, stop: 1 #ffff00);}"
	);

	settingsButton->setStyleSheet(" QPushButton {\
    border: none;\
  	min-height: 80px;\
	min-width: 100px;\
	max-height: 80px;\
	max-width: 100px;\
 }"
	);

	contactsButton->setStyleSheet(" QPushButton {\
    border: none;\
	min-height: 80px;\
	min-width: 100px;\
	max-height: 80px;\
	max-width: 100px;\
 }"
	);

	statusMenu->setStyleSheet("QMenu {\
			margin-left: 0px;\
			margin-top: 0px;\
			margin-right: 0px;\
			margin-bottom: 0px;\
		font: bold 16px; \
 }\
 QMenu::item {\
			margin-left: 0px;\
			margin-top: 0px;\
			margin-right: 0px;\
			margin-bottom: 0px;\
			padding: 0px 10px 0px 40px;\
			min-height: 68px;\
			min-width: 80px;\
			max-height: 68px;\
			border-radius: 5px;\
 }"
	);

	setStyleSheet(
		"QMenu {\
			margin-left: 0px;\
			margin-top: 0px;\
			margin-right: 0px;\
			margin-bottom: 0px;"
		"	color: none; "
		"	font: bold 16px; "
		"	min-width: 180px; "
		"}\
\
		QMenu::item {\
			margin-left: 0px;\
			margin-top: 0px;\
			margin-right: 0px;\
			margin-bottom: 0px;\
			padding: 0px 10px 0px 20px;\
			min-height: 68px;\
			min-width: 180px;\
			max-height: 68px;\
			border-radius: 5px;\
		}\
\
QMenu::item:checked {\
			background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,\
                                       stop: 0 #999900, stop: 0.55 #aaaa00,\
			                           stop: 0.56 #bbbb00, stop: 1 #ffff00);}\
 QMenu::item:disabled {\
			background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,\
                                       stop: 0 #999, stop: 0.55 #aaa,\
                                       stop: 0.56 #bbb, stop: 1 #fff);}\
 QMenu::separator {\
             height: 10px;\
             margin-left: 5px;\
             margin-right: 5px;\
         }"
	);

	_pActiveSessions->setStyleSheet(
		"QPushButton {"
			"font: bold 16px;"
			"border: none;"
			"border-radius: 5px;"
			"min-height: 60px;"
			"min-width: 100px;"
			"max-height: 60px;"
			"max-width: 100px;"
		"}"
	);	
	
	
}


//--------------------------------------------------------
//ActiveStatusesMenu
//--------------------------------------------------------

ActiveSessions::ActiveSessions(QWidget * parent):
	QWidget(parent),
	_visibleActionsAllowed(3),
	_firstVisibleAction(0),
	_lastVisibleAction(0)
{
	_pActiveSessionsMenu = new QMenu( this );
	_pActiveSessionsMenu->setObjectName("activeSessionsMenu");

	_pSeparatorAction = new QAction(this);
	_pSeparatorAction->setSeparator(true);

	_pCloseAllSessionsAction = new QAction( QIcon(":/close.png"), tr("Close all"), this );
	
	CONNECT( _pCloseAllSessionsAction, SIGNAL(triggered()), parent, SLOT(closeAllSessions()) );
	CONNECT( _pActiveSessionsMenu, SIGNAL(triggered(QAction*)), parent, SLOT(onActiveSessionTriggered(QAction*)) );
	
	_pScrollAction = new QWidgetAction(this);
	_pScrollWidget = new ScrollWidget(this);
	_pScrollAction->setDefaultWidget( _pScrollWidget );
	
	CONNECT( _pScrollWidget->scrollUpButton(), SIGNAL(clicked()), this, SLOT(scrollUp()) );
	CONNECT( _pScrollWidget->scrollDownButton(), SIGNAL(clicked()), this, SLOT(scrollDown()) );
}


QAction* ActiveSessions::closeCurrentSession() {

	qDebug()<<__PRETTY_FUNCTION__;
	
	QAction* pAction = disabledAction();
	
	if( pAction ) 
	{
		removeAction( pAction );
				
		QAction* pNewCurrentAction = 0;

		
		if( _actionsList.count() > 0 ) 
		{
			pNewCurrentAction = _actionsList.last();
			pNewCurrentAction->setEnabled( false );
		}
		
		if( _actionsList.size() == 1 ) 
		{
			_pActiveSessionsMenu->removeAction( _pCloseAllSessionsAction );
			_pActiveSessionsMenu->removeAction( _pSeparatorAction );
		}

		return pNewCurrentAction;
	}
	else 
		return 0;
}


void ActiveSessions::clear() {
		
	_pActiveSessionsMenu->clear();
	_actionsList.clear();
} 

QList<QAction*> ActiveSessions::actions() const {
	
	return _pActiveSessionsMenu->actions();
}

QAction* ActiveSessions::disabledAction() const {
		
	foreach( QAction* pAction, _actionsList ) {
	
		if( !pAction->isEnabled() )
			return pAction;
	}
	return 0;
}

QAction* ActiveSessions::action( const QString& uid ) const {
	
	if(uid.isEmpty()) return 0;
	
	QList<QAction*> actionsList = _pActiveSessionsMenu->actions();
	
	foreach( QAction* pAction, actionsList ) {
		
		if( pAction->data().toString() == uid )
			return pAction;
	}
	return 0;
}

void ActiveSessions::onContactPresenceUpdate(const QString& uid, Presence presence) {
	
	if( QAction* pAction = action( uid ) ) {
		
		switch( presence ) {
		
			case Available:
				pAction->setIcon( QIcon(":/online.png") );
				break;
			case XA:
			case Away:
				pAction->setIcon( QIcon(":/away.png") );
				break;
			case Busy:
				pAction->setIcon( QIcon(":/busy.png") );
				break;
			case Hidden:
				pAction->setIcon( QIcon(":/hidden.png") );
				break;
			case Offline:
				pAction->setIcon( QIcon(":/offline.png") );
				break;
			default:
				pAction->setIcon( QIcon() );
		}
	}
}

void ActiveSessions::onNewMessage(const QString& uid, const QString& name, Presence presence) {

	//If such action already exists - set it checked
	if( QAction* pAction = action( uid ) ) {
		
		if( pAction != disabledAction() )
			pAction->setChecked( true );
						
	} else { //create a new one
		
		QIcon icon;

		switch( presence ) {
		
			case Available:
				icon = QIcon(":/online.png");
				break;
			case XA:
			case Away:
				icon = QIcon(":/away.png");
				break;
			case Busy:
				icon = QIcon(":/busy.png");
				break;
			case Hidden:
				icon = QIcon(":/hidden.png");
				break;
			case Offline:
				icon = QIcon(":/offline.png");
				break;
		}

		pAction = new QAction( icon, name, this );
		pAction->setData( QVariant::fromValue( uid ) );
		pAction->setCheckable( true );
		pAction->setChecked( true );
		
		addAction( pAction );

		
		if( _pActiveSessionsMenu->actions().count() == 2) {

			_pActiveSessionsMenu->addAction( _pSeparatorAction );
			_pActiveSessionsMenu->addAction( _pCloseAllSessionsAction );
		}
	}

}
	
void ActiveSessions::onNewSession(const QString& uid, const QString& name, Presence presence) {

	QAction* pDisabledAction = disabledAction(); 
	
	if( QAction* pAction = action( uid ) ) {

		if( pAction != pDisabledAction ) {

			if(pDisabledAction)
				pDisabledAction->setEnabled( true );
			
			pAction->setChecked( false );
			pAction->setEnabled( false );
		}

	} else {

		QIcon icon;
		switch( presence ) {
			
			case Available:
				icon = QIcon(":/online.png");
				break;
			case XA:
			case Away:
				icon = QIcon(":/away.png");
				break;
			case Busy:
				icon = QIcon(":/busy.png");
				break;
			case Hidden:
				icon = QIcon(":/hidden.png");
				break;
			case Offline:
				icon = QIcon(":/offline.png");
				break;
			default: 
				icon = QIcon(":/offline.png");
		}
		
		if( pDisabledAction )
			pDisabledAction->setEnabled( true );
		
		pAction = new QAction( icon, name, this );
		pAction->setData( QVariant::fromValue( uid ) );
		pAction->setCheckable( true );
		pAction->setEnabled( false );
		
		addAction( pAction );

		_pActiveSessionsMenu->setEnabled(true);
		

		if( _pActiveSessionsMenu->actions().count() == 2) {
			
			_pActiveSessionsMenu->addAction( _pSeparatorAction );
			_pActiveSessionsMenu->addAction( _pCloseAllSessionsAction );
		}

	}	
}

void ActiveSessions::addAction(QAction *pAction) {
		
	QList<QAction*> currentlyVisibleActions = _pActiveSessionsMenu->actions();
	
	if( currentlyVisibleActions.count( _pScrollAction ) )
		_pActiveSessionsMenu->insertAction( _pScrollAction, pAction );

	else if( currentlyVisibleActions.count( _pSeparatorAction ) )
		_pActiveSessionsMenu->insertAction( _pSeparatorAction, pAction );
	else
		_pActiveSessionsMenu->addAction( pAction );
	
	_actionsList.append(pAction);
	
	//set iterators
	_lastVisibleAction = _actionsList.end() - 1;

	if( _actionsList.size() > _visibleActionsAllowed )
		_firstVisibleAction = _actionsList.end() - _visibleActionsAllowed;
	else
		_firstVisibleAction = _actionsList.begin();

	
	if( _actionsList.size() > _visibleActionsAllowed )
	{
		//add scroll action
		_pActiveSessionsMenu->insertAction( _pSeparatorAction, _pScrollAction );	
		
		//remove one action from top to keep constant menu size
		if( _actionsList.size() > _visibleActionsAllowed )
		{
			QAction *pRemoveMe = *( _firstVisibleAction - 1 );
			if(pRemoveMe)
				_pActiveSessionsMenu->removeAction( pRemoveMe );
		}
		
		checkScrollButtonsState();
	}

}

void ActiveSessions::removeAction(QAction *pAction) {
		
	_actionsList.removeOne(pAction);
		
	_lastVisibleAction = _actionsList.end() - 1;
	
	//Reset iterators
	if( _actionsList.size() > _visibleActionsAllowed )
		_firstVisibleAction = _actionsList.end() - _visibleActionsAllowed;
	else
		_firstVisibleAction = _actionsList.begin();

	//Remove all stuff above _pScrollAction or _pSeparatorAction
	QList<QAction*> visibleItems = _pActiveSessionsMenu->actions();
	QList<QAction*>::iterator it = visibleItems.begin();

	while( it != visibleItems.end() )
	{
		if( *it == _pScrollAction || *it == _pSeparatorAction) break;
			
		_pActiveSessionsMenu->removeAction((*it));
		
		++it;
	}
	
	//...and add it back again
	it = _firstVisibleAction;
	
	while( it != (_lastVisibleAction + 1) )
	{
		if( visibleItems.count( _pScrollAction ) )
			_pActiveSessionsMenu->insertAction( _pScrollAction, *it );

		else if( visibleItems.count( _pSeparatorAction ) )
			_pActiveSessionsMenu->insertAction( _pSeparatorAction, *it );
		else
			_pActiveSessionsMenu->addAction( *it );		
			
		++it;
	}
	
	if( _actionsList.count() <= _visibleActionsAllowed  && _pScrollAction->isVisible())
		_pActiveSessionsMenu->removeAction( _pScrollAction );

	
	if( _actionsList.isEmpty() ) return;

	//Reset visible actions
	QList<QAction*> visibleActions = _pActiveSessionsMenu->actions();
	QAction *pFirstVisible = visibleActions[0];


	_firstVisibleAction = _actionsList.begin() + _actionsList.indexOf(pFirstVisible);
	
	if(_actionsList.count() >= _visibleActionsAllowed )
		_lastVisibleAction = _firstVisibleAction + _visibleActionsAllowed - 1;
	else
		_lastVisibleAction = _firstVisibleAction + _actionsList.size() - 1;
	
	checkScrollButtonsState();
}


void ActiveSessions::scrollDown() {
	
	//Is there something below?
	if( _lastVisibleAction == (_actionsList.end() - 1) ) return;

	_pActiveSessionsMenu->removeAction( *_firstVisibleAction );
	
	//Add an item from the bottom
	if( _pScrollAction->isVisible() )
		_pActiveSessionsMenu->insertAction( _pScrollAction, *(_lastVisibleAction + 1) );
	else
		_pActiveSessionsMenu->addAction( *(_lastVisibleAction + 1) );
	
	//adjust iterators
	++_firstVisibleAction;
	++_lastVisibleAction;
	
	checkScrollButtonsState();
}

void ActiveSessions::scrollUp() {
	
	//Is there something above?
	if( _firstVisibleAction == _actionsList.begin() ) return;

	//Add an item from the top
	_pActiveSessionsMenu->insertAction(*_firstVisibleAction, *(_firstVisibleAction - 1) );
	
	//Hide an item from the bottom
	_pActiveSessionsMenu->removeAction( *_lastVisibleAction );
	
	//adjust iterators
	--_firstVisibleAction;
	--_lastVisibleAction;
	
	checkScrollButtonsState();
}

void ActiveSessions::checkScrollButtonsState() {
	
	if( _firstVisibleAction != _actionsList.begin() )
		_pScrollWidget->scrollUpButton()->setEnabled( true );
	else
		_pScrollWidget->scrollUpButton()->setEnabled( false );
	
	
	if( _lastVisibleAction != _actionsList.end() - 1)
		_pScrollWidget->scrollDownButton()->setEnabled( true );
	else
		_pScrollWidget->scrollDownButton()->setEnabled( false );
}

//--------------------------------------------------------
//	ScrollWidget
//--------------------------------------------------------
ScrollWidget::ScrollWidget(QWidget* parent):
	QWidget(parent)
{
	setObjectName("scrollWidget");
	setContentsMargins( -10, -10, -10, -10 );

	_pLayout = new QHBoxLayout(this);
	_pLayout->setSpacing(0);
	_pScrollUpButton = new QPushButton(tr("up"), this);
	_pScrollUpButton->setObjectName("scrollUpButton");
	_pScrollDownButton = new QPushButton(tr("down"), this);
	_pScrollDownButton->setObjectName("scrollDownButton");
		
	_pLayout->addWidget(_pScrollUpButton);
	_pLayout->addWidget(_pScrollDownButton);
	
	show();
}
