/** \file TocMainWindow.cpp
 * \brief Implementation of TocMainWindow class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "TocMainWindow"

#include "TocMainToolbar"
#include "TocContactListWidget"
#include "TocChatWidget"
#include "TocUserInfoWidget"
#include "TocSettingsWidget"
#include "TocGeneralSettingsWidget"
#include "TocAccountSettingsWidget"
#include "TocAccountIntermediateWidget"
#include "TocThemesSettingsWidget"
#include "TocSettings"
#include "macros.h"
#include "version.h"

#include <QStackedLayout>
#include <QList>
#include <QDebug>
#include <QMessageBox>
#include <QMenuBar>
#include <QFile>
#include <QCloseEvent>


Q_DECLARE_METATYPE(TocContact*);

TocMainWindow::TocMainWindow(TocBaseEngine* engine, QWidget * parent, Qt::WindowFlags flags) :
	TocBaseMainWindow(engine, parent, flags), _register(false) {
	
	setWindowTitle("Tieto Open Communicator");
	setContentsMargins( -20, 0, -20, -5 );
	_pCentralWidget = new QWidget( this );
	_pCentralWidget->setObjectName("_pCentralWidget");

	QAction* pAboutAct = new QAction(tr("About"), this);
	pAboutAct->setStatusTip(tr("Show About info"));
	CONNECT(pAboutAct, SIGNAL(triggered()), this, SLOT(showAbout()));

	QAction* pExitAct = new QAction(tr("Exit"), this);
	pExitAct->setStatusTip(tr("Close application"));
	CONNECT(pExitAct, SIGNAL(triggered()), this, SLOT(close()));

	menuBar()->addAction(pAboutAct);
	menuBar()->addAction(pExitAct);

	setCentralWidget( _pCentralWidget );
	QVBoxLayout* verticalLayout = new QVBoxLayout( _pCentralWidget );
	verticalLayout->setSpacing( 0 );

	_pMainToolbar = new TocMainToolbar( _pCentralWidget );
	CONNECT( _pMainToolbar, SIGNAL(contactsClicked()), this, SLOT(showContacts()) );
	CONNECT( _pMainToolbar, SIGNAL(settingsClicked()), this, SLOT(showSettings()) );
	CONNECT( _pMainToolbar, SIGNAL(finishedSession(const QString&)), this, SLOT(closeChat(const QString&)) );
	CONNECT( _pMainToolbar, SIGNAL(finishedAllSessions()), this, SLOT(closeAllOpenChats()) );
	CONNECT( _pMainToolbar, SIGNAL(activeSessionTriggered(QString,QString)), this, SLOT(showChat(QString,QString)) );
	CONNECT( _pMainToolbar, SIGNAL(presenceUpdate(Presence,QString)), this, SIGNAL(presenceChanged(Presence,QString)) );

	verticalLayout->addWidget( _pMainToolbar );
	_pMainLayout = new QStackedLayout( verticalLayout );
	_pMainLayout->setSpacing( 0 );

	_pChatList = new QList<TocChatWidget*>();
	_pOpenChatList = new QList<TocChatWidget*>();

	_pSettingsWidget = new TocSettingsWidget( _pCentralWidget );
	_pMainLayout->addWidget( _pSettingsWidget );
	CONNECT(_pSettingsWidget, SIGNAL(triggered(int)), this, SLOT(showView(int)));

	_pContactList = new TocContactListWidget( _pCentralWidget );
	_pMainLayout->addWidget( _pContactList );

	CONNECT( _pContactList, SIGNAL(triggered(QString,QString,Presence)), _pMainToolbar, SLOT(onNewSession(QString,QString,Presence)) );
	CONNECT( _pContactList, SIGNAL(contactInfoRequest(QListWidgetItem*)), this, SLOT(showUserInfo(QListWidgetItem*)) );
	CONNECT( _pContactList, SIGNAL(triggered(QString,QString,Presence)), this, SLOT(showChat(QString,QString,Presence)) );
	CONNECT( _pContactList, SIGNAL(addContact(TocContact*)),
			this, SIGNAL(addContact(TocContact*)));
	CONNECT( _pContactList, SIGNAL(removedContact(QString)),
			this, SIGNAL(removeContact(QString)));	
	CONNECT( _pContactList, SIGNAL(editContact(TocContact*,QString)),
			this, SIGNAL(editContact(TocContact*,QString)));
	CONNECT( _pContactList, SIGNAL(displayedNameChanged(QString,QString)),
			this, SLOT(onDisplayedNameChange(QString,QString)) );

	_pUserInfoWidget = 0;
	_pGeneralWidget = 0;
	_pAccountWidget = 0;
	_pIntermediateWidget = 0;
	_pThemesWidget = 0;

	if(isFtu())
		showView( static_cast<int>( FTU ) );
	else
		_pMainLayout->setCurrentWidget( _pContactList );

	setStyleSheet(
//			"TocMainToolbar > QPushButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #aa0, stop: 0.55 #bb0, stop: 0.56 #bb0, stop: 1 #ff0); }"
//			"QPushButton#okButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #0aa, stop: 0.55 #0bb, stop: 0.56 #0bb, stop: 1 #0ff); }"
//			"QPushButton#cancelButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #0aa, stop: 0.55 #0bb, stop: 0.56 #0bb, stop: 1 #0ff); }"
//			"QPushButton#generalButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #0aa, stop: 0.55 #0bb, stop: 0.56 #0bb, stop: 1 #0ff); }"
//			"QPushButton#themesButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #0aa, stop: 0.55 #0bb, stop: 0.56 #0bb, stop: 1 #0ff); }"
//			"QPushButton#accountButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #0aa, stop: 0.55 #0bb, stop: 0.56 #0bb, stop: 1 #0ff); }"
//			"QPushButton#networkButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #0aa, stop: 0.55 #0bb, stop: 0.56 #0bb, stop: 1 #0ff); }"
//			"QPushButton#statusButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #aaa, stop: 0.55 #bbb, stop: 0.56 #bbb, stop: 1 #fff); }"
//			"QPushButton#settingsButton { background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #0aa, stop: 0.55 #0bb, stop: 0.56 #0bb, stop: 1 #0ff); }"
//			"*[mandatoryField=\"true\"] {background-color: qlineargradient(x1: 0, y1: 1, x2: 0, y2: 0,"
//			"stop: 0 #aa0, stop: 0.55 #bb0, stop: 0.56 #bb0, stop: 1 #ff0); }"

			"QMessageBox QPushButton {"
			"border: none;"
			"min-width: 80px;"
			"min-height: 40px;"
			"}"
			"QMenuBar {"
			"max-height: 0px;"
			"}"
	);

	_defaultStyle = styleSheet();
	reloadTheme();
}

TocMainWindow::~TocMainWindow() {

	delete _pChatList;
	delete _pOpenChatList;
}

TocChatWidget* TocMainWindow::chatForUid(const QString& uid) {
	foreach( TocChatWidget* chat, *_pChatList )
		if ( chat->uid() == uid )
			return chat;
	return 0;
}

TocChatWidget* TocMainWindow::openChatForUid(const QString& uid) {
	foreach( TocChatWidget* chat, *_pOpenChatList )
		if ( chat->uid() == uid )
			return chat;
	return 0;
}

void TocMainWindow::onPresenceUpdate(Presence presence, QString desc) {
	_pMainToolbar->onPresenceUpdate( presence, desc );
	if( presence == Offline )
		_pContactList->clearStatuses();
}

void TocMainWindow::onContactPresenceUpdate(QString uid, Presence presence, QString desc) {
	if ( TocChatWidget* chat = openChatForUid( uid ) ) {
		chat->onContactPresenceUpdate( presence, desc );
		_pMainToolbar->onContactPresenceUpdate( uid, presence );
	}	

	_pContactList->onContactPresenceUpdate( uid, presence, desc );
}

void TocMainWindow::onIncomingMessage(QString uid, Message message) {

	TocChatWidget* chat;
	if ( chat = chatForUid( uid ) ) {
		_pOpenChatList->append( chat );
		chat->appendMessage( message );
		QListWidgetItem* pItem = _pContactList->findItem( uid );
		if( pItem ) {
			QString name = pItem->data( DisplayRole ).toString();
			TocContact *pContact = pItem->data(ContactRole).value<TocContact*>();
			_pMainToolbar->onNewMessage( uid, name, pContact->presence );
		} else {
			_pMainToolbar->onNewMessage( uid, uid, static_cast<Presence>(0) );
		}
	} else {
		QListWidgetItem* pItem = _pContactList->findItem( uid );
		if( !pItem ) {
			chat = new TocChatWidget( uid, uid, static_cast<Presence>(0), _pCentralWidget, message );
			_pChatList->append( chat );
			_pOpenChatList->append( chat );
			_pMainLayout->addWidget( chat );
			_pMainToolbar->onNewMessage( uid, uid, static_cast<Presence>(0) );
		} else {
			QString name = pItem->data( DisplayRole ).toString();
			TocContact *pContact = pItem->data(ContactRole).value<TocContact*>();
			chat = new TocChatWidget( uid, name, pContact->presence, _pCentralWidget, message );
			_pChatList->append( chat );
			_pOpenChatList->append( chat );
			_pMainLayout->addWidget( chat );
			_pMainToolbar->onNewMessage( uid, name, pContact->presence );
		}
		CONNECT(chat, SIGNAL(newMessageReady(QString,Message)), this, SIGNAL(newMessageReady(QString,Message)));
	}
}

void TocMainWindow::closeChat(const QString& uid) {

	if ( TocChatWidget* chat = openChatForUid( uid ) ) {
		_pOpenChatList->removeOne( chat );
		emit sessionClosed( uid );
	}
}

void TocMainWindow::closeAllOpenChats() {

	foreach( TocChatWidget* chat, *_pOpenChatList ) {
		emit sessionClosed( chat->uid() );
	}
	_pOpenChatList->clear();
}

void TocMainWindow::clearAllChats() {

	// First close open chats
	closeAllOpenChats();

	foreach( TocChatWidget* chat, *_pChatList ) {
		_pMainLayout->removeWidget( chat );
		delete chat;
	}

	_pChatList->clear();
}

void TocMainWindow::showChat(const QString& uid, const QString& name, Presence presence) {

	if ( TocChatWidget* chat = chatForUid( uid ) ) {
		if(!openChatForUid(uid)) {
			_pOpenChatList->append( chat );
			chat->updateContactPresence( presence );
		}

		_pMainLayout->setCurrentWidget( chat );
	} else {
		chat = new TocChatWidget( uid, name, presence, _pCentralWidget );
		_pChatList->append( chat );
		_pOpenChatList->append( chat );
		_pMainLayout->addWidget( chat );
		_pMainLayout->setCurrentWidget( chat );
		CONNECT(chat, SIGNAL(newMessageReady(QString,Message)), this, SIGNAL(newMessageReady(QString,Message)));
	}
}

void TocMainWindow::showContacts() {
	_pMainLayout->setCurrentWidget( _pContactList );
}

void TocMainWindow::returnToContacts() {
	_pMainLayout->setCurrentWidget( _pContactList );
	_pMainToolbar->onCurrentView( Contacts );
}

void TocMainWindow::showSettings() {
	_pMainLayout->setCurrentWidget( _pSettingsWidget );
}

void TocMainWindow::returnToSettings() {
	_pMainLayout->setCurrentWidget( _pSettingsWidget );
	_pMainToolbar->onCurrentView( Settings, tr("Settings") );
}

void TocMainWindow::reloadTheme() {
	QString current = styleSheet();
	QString theme;

	TocSettings* pSettings = TocSettings::getInstance();

	QFile file(pSettings->currentTheme());

	file.open(QIODevice::ReadOnly | QIODevice::Text);
	if (file.exists()) {
		theme = file.readAll();
		file.close();
	} else
		return;

	if (theme + _defaultStyle != current) {
		if (theme.isEmpty())
			setStyleSheet(_defaultStyle);
		else
			setStyleSheet(theme + _defaultStyle);
	}
}

void TocMainWindow::returnToIntermediate() {
	if(isFtu())
		showView( static_cast<int>( FTU ) );
	else
		showView( static_cast<int>( AccountIntermediate ) );
}

void TocMainWindow::showAccount() {
	showView( static_cast<int>( Account ) );
}

void TocMainWindow::showAccountCreate() {
	showView( static_cast<int>( AccountCreate ) );
}

void TocMainWindow::showView(int type) {
	ViewType viewType = static_cast<ViewType>( type );

	switch( viewType ) {

	case Contacts:
		showContacts();
		_pMainToolbar->onCurrentView( Contacts );
		break;

	case Chat:
		/* Do nothing */
		break;

	case UserInfo:
		showUserInfo( 0 );
		break;

	case Settings:
		showSettings();
		break;

	case General:
		if( !_pGeneralWidget ) {
			_pGeneralWidget = new TocGeneralSettingsWidget( _pCentralWidget );
			CONNECT(_pGeneralWidget, SIGNAL(backClicked()), this, SLOT(returnToSettings()));
			CONNECT(_pGeneralWidget, SIGNAL(finished()), this, SLOT(returnToContacts()));
			_pMainLayout->addWidget( _pGeneralWidget );
		}
		_pMainToolbar->onCurrentView( viewType, tr("General Settings") );
		_pMainLayout->setCurrentWidget( _pGeneralWidget );
		break;

	case FTU:
		if( !_pIntermediateWidget ) {
			_pIntermediateWidget = new TocAccountIntermediateWidget( _pCentralWidget );
			CONNECT(_pIntermediateWidget, SIGNAL(backClicked()), this, SLOT(returnToSettings()));
			CONNECT(_pIntermediateWidget, SIGNAL(editClicked()), this, SLOT(showAccount()));
			CONNECT(_pIntermediateWidget, SIGNAL(createClicked()), this, SLOT(showAccountCreate()));
			_pMainLayout->addWidget( _pIntermediateWidget );
		}
		_pIntermediateWidget->enableFTUMode( true );
		_pMainToolbar->onCurrentView( viewType, tr("Account Settings") );
		_pMainLayout->setCurrentWidget( _pIntermediateWidget );
		break;
	
	case AccountIntermediate:
		if( !_pIntermediateWidget ) {
			_pIntermediateWidget = new TocAccountIntermediateWidget( _pCentralWidget );
			CONNECT(_pIntermediateWidget, SIGNAL(backClicked()), this, SLOT(returnToSettings()));
			CONNECT(_pIntermediateWidget, SIGNAL(editClicked()), this, SLOT(showAccount()));
			CONNECT(_pIntermediateWidget, SIGNAL(createClicked()), this, SLOT(showAccountCreate()));
			_pMainLayout->addWidget( _pIntermediateWidget );
		}
		_pIntermediateWidget->enableFTUMode( false );
		_pMainToolbar->onCurrentView( viewType, tr("Account Settings") );
		_pMainLayout->setCurrentWidget( _pIntermediateWidget );
		break;

	case AccountCreate:
		if( !_pAccountWidget ) {
			_pAccountWidget = new TocAccountSettingsWidget( _pCentralWidget );
			CONNECT(_pAccountWidget, SIGNAL(newAccountSettingsCreated()), this, SLOT(onAccountSettingsCreated()));
			CONNECT(_pAccountWidget, SIGNAL(accountSettingsChanged()), this, SLOT(onAccountSettingsChanged()));			CONNECT(_pAccountWidget, SIGNAL(finished()), this, SLOT(returnToContacts()));
			CONNECT(_pAccountWidget, SIGNAL(finished()), this, SLOT(returnToContacts()));
			CONNECT(_pAccountWidget, SIGNAL(backClicked()), this, SLOT(returnToIntermediate()));
			_pMainLayout->addWidget( _pAccountWidget );
		}
		_pAccountWidget->setCreateMode(true);
		_pMainToolbar->onCurrentView( viewType, tr("Create Account") );
		_pMainLayout->setCurrentWidget( _pAccountWidget );
		break;

	case Account:
		if( !_pAccountWidget ) {
			_pAccountWidget = new TocAccountSettingsWidget( _pCentralWidget );
			CONNECT(_pAccountWidget, SIGNAL(newAccountSettingsCreated()), this, SLOT(onAccountSettingsCreated()));
			CONNECT(_pAccountWidget, SIGNAL(accountSettingsChanged()), this, SLOT(onAccountSettingsChanged()));
			CONNECT(_pAccountWidget, SIGNAL(finished()), this, SLOT(returnToContacts()));
			CONNECT(_pAccountWidget, SIGNAL(backClicked()), this, SLOT(returnToIntermediate()));
			_pMainLayout->addWidget( _pAccountWidget );
		}
		_pAccountWidget->setCreateMode(false);
		_pMainToolbar->onCurrentView( viewType, tr("Account Settings") );
		_pMainLayout->setCurrentWidget( _pAccountWidget );
		break;

	case Themes:
		if( !_pThemesWidget ) {
			_pThemesWidget = new TocThemesSettingsWidget( _pCentralWidget );
			_pMainLayout->addWidget( _pThemesWidget );
			CONNECT(_pThemesWidget, SIGNAL(backClicked()), this, SLOT(returnToSettings()));
			CONNECT(_pThemesWidget, SIGNAL(currentThemeChanged()), this, SLOT(reloadTheme()));
		}	
		_pMainToolbar->onCurrentView( viewType, tr("Themes Settings") );
		_pMainLayout->setCurrentWidget( _pThemesWidget );
		break;

	default:
		/* Do nothing */
		break;
	}
}

void TocMainWindow::showUserInfo(QListWidgetItem* item) {
	
	_pMainLayout->removeWidget( _pUserInfoWidget );
	delete _pUserInfoWidget;
	
	_pUserInfoWidget = new TocUserInfoWidget( item, _pContactList );

	if ( item == 0 ) {
		CONNECT(_pUserInfoWidget, SIGNAL(contactChanged(QListWidgetItem*)),
				_pContactList, SLOT(commitAdd(QListWidgetItem*)));

		_pMainToolbar->onCurrentView( UserInfo, tr("Create Contact") );

	} else {
		CONNECT(_pUserInfoWidget, SIGNAL(contactChanged(QListWidgetItem*)),
				_pContactList, SLOT(commitEdit(QListWidgetItem*)));

		QString name = item->data( DisplayRole ).toString();
		_pMainToolbar->onCurrentView( UserInfo, tr("Edit %1").arg( name ) );
	}

	CONNECT(_pUserInfoWidget, SIGNAL(finished()), this, SLOT(returnToContacts()));

	_pMainLayout->addWidget( _pUserInfoWidget );
	_pMainLayout->setCurrentWidget( _pUserInfoWidget );
}

void TocMainWindow::onDisplayedNameChange(const QString& uid, const QString& name) {

	if ( TocChatWidget* chat = chatForUid( uid ) ) {
		chat->onDisplayedNameChange( name );
		_pMainToolbar->onDisplayedNameChange( uid, name );
	}
}

void TocMainWindow::onContactListReceived(const TocContactList& cList)
{
	_pContactList->onNewContactList(cList);
}

void TocMainWindow::onAuthorizationRequest(QString /*uid*/) {
	//TODO: finish implementation
}

void TocMainWindow::onSubscriptionAccepted(QString /*uid*/) {
	//TODO: finish implementation
}

void TocMainWindow::onStatusChange(Status status, Reason reason) {
	switch(status) {
	case Disconnected:
		if(reason != Requested)
			if(_register) {
				QMessageBox::critical(this, tr("TEC"), errorString(reason) + "\nCannot create account");
				_register = false;
			} else
				QMessageBox::critical(this, tr("TEC"), errorString(reason));

		_pMainToolbar->onPresenceUpdate( Offline, "" );

		break;
	case Connected:
		if(_register) {
			QMessageBox::information(this, tr("TEC"), tr("Account created successfully!"));
			_register = false;
			returnToContacts();
		} break;
	case Connecting:
		/* Do nothing */
		break;
	default:
		/* Do nothing */
		break;
	}
}

QString TocMainWindow::errorString(Reason reason) {
	switch(reason) {
	case NoneSpecified:
		return tr("Disconnected because of unknown reason.");

	case Requested:
		return tr("Disconnected because it was requested.");

	case NetworkError:
		return tr("Disconnected because of network error.");

	case AuthenticationFailed:
		return tr("Disconnected because the username or password was invalid.");

	case EncryptionError:
		return tr("Disconnected because of encryption error.");

	case NameInUse:
		return tr("Could not create account or connect because username is already in use.");

	case CertificateNotProvided:
		return tr("Disconnected because certificate was not provided.");

	case CertificateUntrusted:
		return tr("Disconnected because certificate is untrusted.");

	case CertificateExpired:
		return tr("Disconnected because certificate has expired.");

	case CertificateNotActivated:
		return tr("Disconnected because certificate has not been activated.");

	case CertificateHostnameMismatch:
		return tr("Disconnected because of certificate hostname mismatch.");

	case CertificateFingerPrintMismatch:
		return tr("Disconnected because of certificate finger print mismatch.");

	case CertificateSigned:
		return tr("Disconnected because certificate was self-signed.");

	case CertificateOtherError:
		return tr("Disconnected because of non specified certificate error.");

	default:
		return tr("Disconnected because of unknown reason.");
	}
}

bool TocMainWindow::isFtu() {
	TocSettings *set = TocSettings::getInstance();
	if(!set->accountUid().length() && !set->accountPasswd().length())
		return true;
	return false;
}

void TocMainWindow::showAbout() {
	QMessageBox::about(this, tr("About Tieto Open Communicator"), tr(	
				"Tieto Open Communicator %1 - Instant Message Communicator for Telepathy framework\n\n"
				"Copyright: (c) 2010, Tieto Corporation\n"
				"License: see `/usr/share/%2/licence/copyright`."
						).arg(VERSION).arg(TARGET));
}

void TocMainWindow::onAccountSettingsChanged() {
	_register = false;
	emit presenceChanged(Offline, "");

	// Clear chat widgets and active sessions
	clearAllChats();
	_pMainToolbar->clearAllSessions();

	// Empty the contact list
	TocSettings *pSettings = TocSettings::getInstance();
	pSettings->contactList()->clear();
	_pContactList->onNewContactList(*pSettings->contactList());
}

void TocMainWindow::onAccountSettingsCreated() {
	onAccountSettingsChanged();

	_pMainToolbar->onCurrentView(AccountCreate, tr("Create Account"));
	_register = true;
	// Request to create account
	emit presenceChanged(Available, "");
	QMessageBox::information(this, tr("TEC"),
		tr("To be able to fully use Your new account:\nplease reconnect."));
}

void TocMainWindow::closeEvent( QCloseEvent* event ) {
	Q_UNUSED(event)
	hide();
}

int TocMainWindow::top_application() {
    show();
    return 0;
}
