/** \file.cpp
 * \brief Implementation of TocUserInfoWidget class
 * 
 * Tieto Open Communicator - Client for the Telepathy communications framework.
 * Copyright (c) 2010, Tieto Corporation
 *
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 	Redistributions of source code must retain the above copyright notice,
 * 	this list of conditions and the following disclaimer.
 * 	Redistributions in binary form must reproduce the above copyright notice,
 * 	this list of conditions and the following disclaimer in the documentation
 * 	and/or other materials provided with the distribution.
 * 	Neither the name of the Tieto Corporation nor the names of its contributors 
 *	may be used to endorse or promote products derived from this software without
 * 	specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include <QDebug>
#include <QFontMetrics>
#include <TocContactListWidget>
#include <QMessageBox>
#include <QShowEvent>
#include <QListWidgetItem>

#include "TocUserInfoWidget"
#include "IconProvider"
#include "ui_TocUserInfoWidget.h"
#include "uidefs.h"
#include "macros.h"

Q_DECLARE_METATYPE(TocContact*);


TocUserInfoWidget::TocUserInfoWidget(QListWidgetItem * contactItem, TocContactListWidget * parent, Qt::WindowFlags flags)
	: QWidget(parent, flags), _pContactList(parent)
{

	ui = new Ui_TocUserInfoWidgetClass();
	ui->setupUi(this);
	setContentsMargins(-10, -10, -10, -10 );

	ui->resolveButton->hide(); //TODO: implement resolving contact's name functionality

	//Adjust labels size to their content
	QFontMetrics fm(ui->uidLabel->font());

	//Find the widest 
	QList<int> list;
	list << fm.boundingRect(ui->uidLabel->text()).width();
	list << fm.boundingRect(ui->nameLabel->text()).width();
	list << fm.boundingRect(ui->genderLabel->text()).width();
	qSort(list.begin(), list.end());

	//Set minimum width
	ui->uidLabel->setMinimumWidth(list.last());
	ui->nameLabel->setMinimumWidth(list.last());
	ui->genderLabel->setMinimumWidth(list.last());

	_newCustomPicturePath = "";

	_pContactItem = contactItem;

	ui->genderComboBox->addItem(tr("Unknown") );
	ui->genderComboBox->setItemData(0, NotSpecified);

	ui->genderComboBox->addItem(tr("Male") );
	ui->genderComboBox->setItemData(1, Male);

	ui->genderComboBox->addItem(tr("Female"));
	ui->genderComboBox->setItemData(2, Female);	

	CONNECT(ui->genderComboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(onCurrentIndexChanged(int)));
	CONNECT(ui->okButton, SIGNAL(clicked()),this,SLOT(okClicked()));
	CONNECT(ui->cancelButton,SIGNAL(clicked()),this,SIGNAL(finished()));

	_pIconProvider = IconProvider::getInstance();

	if (!contactItem) {
		ui->pictureButton->setIcon(_pIconProvider->matchIcon(NotSpecified));
		return;
	}

	ui->nameLineEdit->setText(contactItem->data(ContactRole).value<TocContact*>()->name);
	ui->uidLineEdit->setText(contactItem->data(ContactRole).value<TocContact*>()->uid);


	Gender gender = contactItem->data(ContactRole).value<TocContact*>()->gender;

	int idx = ui->genderComboBox->findData(gender);

	if (idx >= 0 && idx <= 2) {
		ui->genderComboBox->setCurrentIndex(idx);
		_currentIndex = idx;
	}

	_customPicturePath = _newCustomPicturePath = contactItem->data(ContactRole).value<TocContact*>()->customIconPath;

	// Get picture
	QIcon icon;
	if(_customPicturePath.isEmpty())
		icon = _pIconProvider->matchIcon(gender);
	else
		icon = QIcon(_customPicturePath);

	ui->pictureButton->setIcon(icon);
}

TocUserInfoWidget::~TocUserInfoWidget() {
	delete ui;
}

void TocUserInfoWidget::onCurrentIndexChanged (int index) {

	Gender gender = static_cast<Gender>(ui->genderComboBox->itemData(index).value<int>());

	// Get picture
	QIcon icon;
	if(_newCustomPicturePath.isEmpty())
		icon = _pIconProvider->matchIcon(gender);
	else
		icon = QIcon(_customPicturePath);

	ui->pictureButton->setIcon(icon);
}

void TocUserInfoWidget::okClicked() {

	if(_pContactItem) { // Edit an existing item

		Gender gender;
		QString uid = ui->uidLineEdit->text();

		if(uid.isEmpty()) {
			QMessageBox::warning(this, tr("TEC"), tr("User ID is empty.\nCannot commit changes."));
			return; 
		}

		// Check if UID is not already on contact list
		if(_pContactList->findItem(uid, _pContactItem)) {
			QMessageBox::warning(this, tr("TEC"), tr("The given user ID already exists on Your contact list.\nCannot commit changes."));
			return;
		}

		QString name = ui->nameLineEdit->text();
		int idx = ui->genderComboBox->currentIndex();

		// Check if something has changed
		bool changed = false;
		if(_pContactItem->data(ContactRole).value<TocContact*>()->uid != uid)
			changed = true;
		else if(_pContactItem->text() != name)
			changed = true;
		else if(_currentIndex != idx)
			changed = true;
		else if(_customPicturePath != _newCustomPicturePath)
			changed = true;

		// If yes: save changes and emit signal to notify TocContactListWidget (commitEdit)
		if(changed) {
			gender = static_cast<Gender>(ui->genderComboBox->itemData(idx).value<int>());

			//Get contact from QListWidgetItem
			TocContact *pContact = _pContactItem->data(ContactRole).value<TocContact*>();

			if(name.isEmpty())
				_pContactItem->setText(uid);
			else
				_pContactItem->setText(name);

			pContact->uid = uid;
			pContact->name = name;
			pContact->gender = gender;

			_pContactItem->setData(Qt::DecorationRole, *(_pIconProvider->matchIcon(pContact)));
			emit contactChanged(_pContactItem);
		}
	} else { // Create new item

		// Collect data for the new contact
		// Get uid
		QString uid = ui->uidLineEdit->text();

		if(uid.isEmpty()) {
			QMessageBox::warning(this, tr("TEC"), tr("User ID is empty.\nCannot commit changes."));
			return; 
		}

		// Check if UID is not already on contact list
		if(_pContactList->findItem(uid)) {
			QMessageBox::warning(this, tr("TEC"), tr("The given user ID already exists on Your contact list.\nCannot commit changes."));
			return;
		}

		// Get name
		QString name = ui->nameLineEdit->text();

		// ... and gender
		int idx = ui->genderComboBox->currentIndex();
		Gender gender = static_cast<Gender>(ui->genderComboBox->itemData(idx).value<int>());		

		// Create QListWidgetItem and emit it to notify TocContactListWidget (commitAdd)
		QString displayedText;
		if(name.isEmpty())
			displayedText = uid;
		else
			displayedText = name;
		
		QListWidgetItem* pNewItem = new QListWidgetItem(displayedText, 0);

		// Create new  contact structure...
		TocContact* pNewContact = new TocContact;

		pNewContact->uid = uid;
		pNewContact->name = name;
		pNewContact->gender = gender;
		pNewContact->customIconId = 0;
		pNewContact->customIconPath = "";
		pNewContact->presence = Offline;

		// ... and associate it with QListWidgetItem
		pNewItem->setData(ContactRole, QVariant::fromValue(pNewContact));

		pNewItem->setData(Qt::DecorationRole, *(_pIconProvider->matchIcon(pNewContact)));
		pNewItem->setFlags(Qt::ItemIsEnabled);

		emit contactChanged(pNewItem);
	}

	// Emit this to finish the job
	emit finished();	
}

void TocUserInfoWidget::showEvent(QShowEvent* event) {
	ui->uidLineEdit->setFocus();
	event->accept();
}
